#include "LLSSolver.h"

//   solving a non-square system of linear equations
//   with singular value decomposition
//   a library from lapack


LLSSolver::LLSSolver(){
}


void LLSSolver::solve(int matrixSizeM, 
		      int matrixSizeN, 
		      real *A, 
		      real *b, 
		      real *x,
		      vector<real> &UFixedParticles,
		      vector<real> &SFixedParticles,
		      vector<real> &VFixedParticles,
		      int &fixedParticles) {

  // note: m > n
  m = matrixSizeM;
  n = matrixSizeN;

  U = allocate1dArray(m*m);
  S = allocate1dArray(n);
  V = allocate1dArray(n*n);
  tmp = allocate1dArray(m);
  
  //  initialize(matrixSizeM, matrixSizeN); // set parameters and allocate memory

  if (fixedParticles == 0) {

    // modify the value of fixedParticles
    fixedParticles = 1;

    calculateUSV(A);

    // assign memories for U, S, V (FixedParticles)
    // store values for them 

    UFixedParticles.assign(m*m, 0.0); 
    SFixedParticles.assign(n, 0.0); 
    VFixedParticles.assign(n*n, 0.0); 
    for (i=0; i<m*m; i++){
      UFixedParticles[i] = U[i];
    }
    for (i=0; i<n; i++){
      SFixedParticles[i] = S[i];
    }
    for (i=0; i<n*n; i++){
      VFixedParticles[i] = V[i];
    }
    
  } else if (fixedParticles == 1){

    for (i=0; i<m*m; i++){
      U[i] = UFixedParticles[i];
    }
    for (i=0; i<n; i++){
      S[i] = SFixedParticles[i];
    }
    for (i=0; i<n*n; i++){
      V[i] = VFixedParticles[i];
    }
    
  }
  
#ifdef SINGLE
  ssol_(U,S,V,&m,&n,b,x,tmp);
#else
  dsol_(U,S,V,&m,&n,b,x,tmp);
#endif	   

  releaseMemory();
}

/*
void LLSSolver::initialize(int matrixSizeM, int matrixSizeN){



}
*/

void LLSSolver::calculateUSV(real *A){

  real *WORK;
  int LWORK, INFO;
  LWORK = max( 3*min(m,n)+max(m,n),5*min(m,n)-4 )+100;
  WORK = allocate1dArray(LWORK);

  clock_t tOld, tNew;		 

  tOld = clock();

  // note: V is actually VT, a transpose
#ifdef SINGLE
  ssvd_(A,U,V,&m,&n,S,WORK,&LWORK,&INFO);
#else
  dsvd_(A,U,V,&m,&n,S,WORK,&LWORK,&INFO);
#endif

  tNew = clock();
  
  cout << "SVD part: " << (tNew - tOld) / (double) CLOCKS_PER_SEC << " secs." << endl ;


  // release memory
  delete [] WORK;
  if(INFO!=0){
    fprintf(stderr,"svd failed\n");
    exit(1);
  }

}

real * LLSSolver::allocate1dArray(int size){
  real *x = new real[size];
  return x;
}

void LLSSolver::delete1dArray(real * x){
  delete [] x;
}

void LLSSolver::releaseMemory( ){

  delete1dArray(S);
  delete1dArray(U);
  delete1dArray(V);
  delete1dArray(tmp);
}

LLSSolver::~LLSSolver(){

}
