#include "boundaryCond.h"


boundaryCond::boundaryCond(){
}

void boundaryCond::initialize(int option, // 1 for uStar and 2 for uNew
			      real t,
			      real dt,
			      int ** pointNumberMeshU, 
			      int ** pointNumberMeshV, 
			      int ** pointNumberMeshW, 
			      real ***xU, 
			      real ***xV, 
			      real ***xW, 
			      real ****u, // boundary conditions for u
			      real ****v,
			      real ****w, 
			      real ****uAux, // boundary conditions for uAux
			      real ****vAux,
			      real ****wAux){

  ///////////////////////////////////////////
  //
  // boundary points in the coarse mesh for u_x, v_y, w_z = 0
  // and specified velocity
  //
  //////////////////////////////////////////

  //west
  i = 0;
  for (k=1; k <= pointNumberMeshU[0][2] - 2; k++) {
    for (j=1; j <= pointNumberMeshU[0][1] - 2; j++) {

      if ( option == 1 ) { // for uStar
	uAux[0][i][j][k] = 0.0; // fixed u = 0

      } else if ( option == 2 ){ // for uNew
	u[0][i][j][k] = 0.0; // fixed u = 0
      }

    }
  }

  //east
  i = pointNumberMeshU[0][0] - 1;  
  for (k=1; k <= pointNumberMeshU[0][2] - 2; k++) {
    for (j=1; j <= pointNumberMeshU[0][1] - 2; j++) {

      if ( option == 1 ) { // for uStar
	//	evolutionary boundary condition
	uAux[0][i][j][k] = uAux[0][i-1][j][k];      
      } else if ( option == 2 ){ // for uNew
	u[0][i][j][k] = u[0][i-1][j][k]; // fixed u = 0
      }

    }
  }

  //south
  j = 0;
  for (k=1; k<=pointNumberMeshV[0][2] - 2; k++) {
    for (i=1; i <= pointNumberMeshV[0][0] - 2; i++) {
 
      if ( option == 1 ) { // for uStar
	vAux[0][i][j][k] = -vAux[0][i][pointNumberMeshV[0][1] - 2][k]; //periodic
      } else if ( option == 2 ){ // for uNew
	v[0][i][j][k] = -v[0][i][pointNumberMeshV[0][1] - 2][k];  //periodic
      }

    }
  }

  //north
  j = pointNumberMeshV[0][1] - 1;
  for (k=1; k<=pointNumberMeshV[0][2] - 2; k++) {
    for (i=1; i <= pointNumberMeshV[0][0] - 2; i++) {
 
      if ( option == 1 ) { // for uStar
	vAux[0][i][j][k] = -vAux[0][i][1][k]; //periodic
      } else if ( option == 2 ){ // for uNew
	v[0][i][j][k] = -v[0][i][1][k];  //periodic
      }

    }
  }

  //bottom
  k = 0;
  for (j=1; j<=pointNumberMeshW[0][1] - 2; j++) {
    for (i=1; i <= pointNumberMeshW[0][0] - 2; i++) {

      if ( option == 1 ) { // for uStar
	wAux[0][i][j][k] = wAux[0][i][j][1]; //periodic
      } else if ( option == 2 ){ // for uNew
	w[0][i][j][k] = w[0][i][j][1]; //periodic
      }
    }
  }



  //top
  k = pointNumberMeshW[0][2] - 1;
  for (j=1; j<=pointNumberMeshW[0][1] - 2; j++) {
    for (i=1; i <= pointNumberMeshW[0][0] - 2; i++) {

      if ( option == 1 ) { // for uStar
	wAux[0][i][j][k] = wAux[0][i][j][k-1]; //periodic
      } else if ( option == 2 ){ // for uNew
	w[0][i][j][k] = w[0][i][j][k-1]; //periodic
      }
    }
  }


  ///////////////////////////////////////////
  //
  // boundary points in the coarse mesh in x direction
  //
  //////////////////////////////////////////

  //west
  i=0;
  for (k=1; k<=pointNumberMeshV[0][2] - 2; k++) {
    for (j=1; j<=pointNumberMeshV[0][1] - 2; j++) {
      

      if ( option == 1 ) { // for uStar
	vAux[0][i][j][k] = - vAux[0][1][j][k]; // 

      } else if ( option == 2 ){ // for uNew
	v[0][i][j][k] = - v[0][1][j][k];
      }

    }
  }


  //east
  i = pointNumberMeshV[0][0] - 1;
  for (k=1; k<=pointNumberMeshV[0][2] - 2; k++) {
    for (j=1; j <= pointNumberMeshV[0][1] - 2; j++) {

      if ( option == 1 ) { // for uStar

	vAux[0][i][j][k] = vAux[0][i-1][j][k];

      } else if ( option == 2 ){ // for uNew
	v[0][i][j][k] = v[0][i-1][j][k];
      }

    }
  }

  //west
  i=0;
  for (k=1; k<=pointNumberMeshW[0][2] - 2; k++) {
    for (j=1; j<=pointNumberMeshW[0][1] - 2; j++) {
      
      if ( option == 1 ) { // for uStar
	wAux[0][i][j][k] = - wAux[0][1][j][k];  // w = 0 
      } else if ( option == 2 ){ // for uNew
	w[0][i][j][k] = - w[0][1][j][k];  //specified w = 0
      }

    }
  }

  //east
  i = pointNumberMeshW[0][0] - 1;
  for (k=1; k<=pointNumberMeshW[0][2] - 2; k++) {
    for (j=1; j <= pointNumberMeshW[0][1] - 2; j++) {

      if ( option == 1 ) { // for uStar

	wAux[0][i][j][k] = wAux[0][i-1][j][k];

      } else if ( option == 2 ){ // for uNew
	w[0][i][j][k] = w[0][i-1][j][k];
      }

    }
  }

  ///////////////////////////////////////////
  //
  // boundary points in the coarse mesh in y direction
  //
  //////////////////////////////////////////


  //south
  j = 0;
  for (k=1; k <= pointNumberMeshU[0][2] - 2; k++) {
    for (i=1; i <= pointNumberMeshU[0][0] - 2; i++) {

      if ( option == 1 ) { // for uStar
	uAux[0][i][j][k] = uAux[0][i][pointNumberMeshU[0][1] - 2][k];
      } else if ( option == 2 ){ // for uNew
	u[0][i][j][k] = u[0][i][pointNumberMeshU[0][1] - 2][k];
      }

    }
  }


  //north
  j = pointNumberMeshU[0][1] - 1;
  for (k=1; k <= pointNumberMeshU[0][2] - 2; k++) {
    for (i=1; i <= pointNumberMeshU[0][0] - 2; i++) {

      if ( option == 1 ) { // for uStar
	uAux[0][i][j][k] = uAux[0][i][1][k];
      } else if ( option == 2 ){ // for uNew
	u[0][i][j][k] = u[0][i][1][k];
      }
    }
  }


  //south
  j=0;
  for (i=1; i<=pointNumberMeshW[0][0] - 2; i++) {
    for (k=1; k<=pointNumberMeshW[0][2] - 2; k++) {

      if ( option == 1 ) { // for uStar
	wAux[0][i][j][k] = wAux[0][i][pointNumberMeshW[0][1] - 2][k];
      } else if ( option == 2 ){ // for uNew
	w[0][i][j][k] = w[0][i][pointNumberMeshW[0][1] - 2][k];
      }

    }
  }

  //north
  j = pointNumberMeshW[0][1] - 1;
  for (i=1; i<=pointNumberMeshW[0][0] - 2; i++) {
    for (k=1; k <= pointNumberMeshW[0][2] - 2; k++) {

      if ( option == 1 ) { // for uStar
	wAux[0][i][j][k] = wAux[0][i][1][k];
      } else if ( option == 2 ){ // for uNew
	w[0][i][j][k] = w[0][i][1][k];
      }

    }
  }


  ///////////////////////////////////////////
  //
  // boundary points in the coarse mesh in z direction
  //
  //////////////////////////////////////////


  //bottom
  k = 0;
  for (j=1; j <= pointNumberMeshU[0][1] - 2; j++) {
    for (i=1; i <= pointNumberMeshU[0][0] - 2; i++) {

      if ( option == 1 ) { // for uStar
	uAux[0][i][j][k] = uAux[0][i][j][1];
      } else if ( option == 2 ){ // for uNew
	u[0][i][j][k] = u[0][i][j][1];
      }

    }
  }

  //top
  k = pointNumberMeshU[0][2] - 1;
  for (j=1; j <= pointNumberMeshU[0][1] - 2; j++) {
    for (i=1; i <= pointNumberMeshU[0][0] - 2; i++) {
      
      if ( option == 1 ) { // for uStar
	uAux[0][i][j][k] = uAux[0][i][j][k-1];
      } else if ( option == 2 ){ // for uNew
	u[0][i][j][k] = u[0][i][j][k-1];
      }

    }
  }

  //bottom
  k=0;
  for (i=1; i<=pointNumberMeshV[0][0] - 2; i++) {
    for (j=1; j<=pointNumberMeshV[0][1] - 2; j++) {

      if ( option == 1 ) { // for uStar
	vAux[0][i][j][k] = vAux[0][i][j][1];
      } else if ( option == 2 ){ // for uNew
	v[0][i][j][k] = v[0][i][j][1];
      }

    }
  }

  //top
  k = pointNumberMeshV[0][2] - 1;
  for (i=1; i<=pointNumberMeshV[0][0] - 2; i++) {
    for (j=1; j <= pointNumberMeshV[0][1] - 2; j++) {

      if ( option == 1 ) { // for uStar
	vAux[0][i][j][k] = vAux[0][i][j][k-1];
      } else if ( option == 2 ){ // for uNew
	v[0][i][j][k] = v[0][i][j][k-1];
      }

    }
  }


  ///////////////////////////////////////////
  //
  // boundary points in the coarse mesh for u_x, v_y, w_z = 0
  // and specified velocity: for corner points
  //
  //////////////////////////////////////////

  //west
  i = 0;
  for (k=0; k <= pointNumberMeshU[0][2] - 1; k++) {
    for (j=0; j <= pointNumberMeshU[0][1] - 1; j++) {

      if ( option == 1 ) { // for uStar
	uAux[0][i][j][k] = 0.0; // fixed u = 0

      } else if ( option == 2 ){ // for uNew
	u[0][i][j][k] = 0.0; // fixed u = 0
      }

    }
  }

  //east
  i = pointNumberMeshU[0][0] - 1;  
  for (k=0; k <= pointNumberMeshU[0][2] - 1; k++) {
    for (j=0; j <= pointNumberMeshU[0][1] - 1; j++) {

      if ( option == 1 ) { // for uStar
	//	evolutionary boundary condition
	uAux[0][i][j][k] = uAux[0][i-1][j][k];      
      } else if ( option == 2 ){ // for uNew
	u[0][i][j][k] = u[0][i-1][j][k]; // fixed u = 0
      }

    }
  }

  //south
  j = 0;
  for (k=0; k<=pointNumberMeshV[0][2] - 1; k++) {
    for (i=0; i <= pointNumberMeshV[0][0] - 1; i++) {
 
      if ( option == 1 ) { // for uStar
	vAux[0][i][j][k] = -vAux[0][i][pointNumberMeshV[0][1] - 2][k]; //periodic
      } else if ( option == 2 ){ // for uNew
	v[0][i][j][k] = -v[0][i][pointNumberMeshV[0][1] - 2][k];  //periodic
      }

    }
  }

  //north
  j = pointNumberMeshV[0][1] - 1;
  for (k=0; k<=pointNumberMeshV[0][2] - 1; k++) {
    for (i=0; i <= pointNumberMeshV[0][0] - 1; i++) {
 
      if ( option == 1 ) { // for uStar
	vAux[0][i][j][k] = -vAux[0][i][1][k]; //periodic
      } else if ( option == 2 ){ // for uNew
	v[0][i][j][k] = -v[0][i][1][k];  //periodic
      }

    }
  }

  //bottom
  k = 0;
  for (j=0; j<=pointNumberMeshW[0][1] - 1; j++) {
    for (i=0; i <= pointNumberMeshW[0][0] - 1; i++) {

      if ( option == 1 ) { // for uStar
	wAux[0][i][j][k] = wAux[0][i][j][1]; //periodic
      } else if ( option == 2 ){ // for uNew
	w[0][i][j][k] = w[0][i][j][1]; //periodic
      }
    }
  }



  //top
  k = pointNumberMeshW[0][2] - 1;
  for (j=0; j<=pointNumberMeshW[0][1] - 1; j++) {
    for (i=0; i <= pointNumberMeshW[0][0] - 1; i++) {

      if ( option == 1 ) { // for uStar
	wAux[0][i][j][k] = wAux[0][i][j][k-1]; //periodic
      } else if ( option == 2 ){ // for uNew
	w[0][i][j][k] = w[0][i][j][k-1]; //periodic
      }
    }
  }



}


boundaryCond::~boundaryCond(){
}
