#ifndef BOUNDARYCOND_H
#define BOUNDARYCOND_H

/** \file boundaryCond.cpp
    \brief Set outer boundary conditions for velocity

    This class sets theboundary conditions for three velocity components in the six outer faces. These include the Direchilet, Neumann, and periodic boundary conditions. Their corresponding boundary conditions for pressure are set in the class pressure.

*/

#include <iostream>
#include <string>
#include <cmath>
using namespace std;

#ifdef SINGLE
#define real float
#else
#define real double
#endif

/** 
    \brief Set outer boundary conditions for velocity

    This class sets theboundary conditions for three velocity components in the six outer faces. These include the Direchilet, Neumann, and periodic boundary conditions. Their corresponding boundary conditions for pressure are set in the class pressure.

*/

class boundaryCond {

 public:
  /**
     constructor
   */
  boundaryCond();


/** 
    Sets the boundary conditions for three velocity components in the six outer faces. These include the Direchilet, Neumann, and periodic boundary conditions.

    set velocity boundary conditions at boundary points in the coarse mesh for west, east, south north, bottom, and top faces.

*/  
  void initialize(int option, /**< 1 for uStar and 2 for uNew */
		  real t, /**< time t */
		  real dt, /**< time increment dt */
		  int ** pointNumberMeshU, /**< mesh for velocity u */
		  int ** pointNumberMeshV, /**<  mesh for velocity v */
		  int ** pointNumberMeshW, /**< mesh for velocity w */
		  real ***xU, /**< coordinates for u */
		  real ***xV, /**< coordinates for v */
		  real ***xW, /**< coordinates for w */
		  real ****u, /**< velocity u in uNew*/
		  real ****v, /**< velocity v in uNew */
		  real ****w, /**< velocity w in uNew */
		  real ****uAux, /**< velocity u in uStar */
		  real ****vAux, /**< velocity v in uStar */
		  real ****wAux /**< velocity w in uStar */
		  );

  /**
     destructor
   */
  ~boundaryCond();
  
 private: 


  // iRHS for index in matrix A = totalUnknowns x totalUnknowns
  // jRHS
 int i, j, k;


  // nx and ny for wavenumber in x and y directions
  int nx, ny, nz;


};

#endif
