#include "cage.h"

// note that this file is not related with m=0,
// the base mesh

cage::cage(){
}

void cage::solve(int numberOfCage,
		 int cellNumberPerPatch,
		 int dimension,
		 int nTruncate,
		 real ReynoldsNumber,
		 real *xParticleCenter, 
		 real *yParticleCenter,
		 real *zParticleCenter,
		 // for p
		 real ***xP,
		 real ****p,
		 int **pointNumberMeshP,
		 int  ****pCagePoint,
		 // for u
		 real ***xU,
		 real ****u,
		 int **pointNumberMeshU,
		 int  ****uCagePoint,
		 // for v
		 real ***xV,
		 real ****v,
		 int **pointNumberMeshV,
		 int  ****vCagePoint,
		 // for w
		 real ***xW,
		 real ****w,
		 int **pointNumberMeshW,
		 int  ****wCagePoint,
		 // for omegaX
		 real ***xOmegaX,
		 real ****omegaX,
		 int **pointNumberMeshOmegaX,
		 int  ****omegaXCagePoint,
		 // for omegaY
		 real ***xOmegaY,
		 real ****omegaY,
		 int **pointNumberMeshOmegaY,
		 int  ****omegaYCagePoint,
		 // for omegaZ
		 real ***xOmegaZ,
		 real ****omegaZ,
		 int **pointNumberMeshOmegaZ,
		 int  ****omegaZCagePoint,
		 real **solvedABCoeffAllParticleNew,
		 vector<real> &UFixedParticles,
		 vector<real> &SFixedParticles,
		 vector<real> &VFixedParticles,
		 int &fixedParticles,
		 int rotatingParticles,
		 real **angVel,
		 real **angVelAcc,
		 real **torqueOld,
		 real **torqueNew){
  
  nCage = numberOfCage;
  nMeshes = nCage + 1;  //need to modify for patch merging
  nCellPerPatch = cellNumberPerPatch;
  dim = dimension;
  Nc = nTruncate;
  Re = ReynoldsNumber;

  // set indices of variables in the cages

  //for p
  indexPCage = allocate4dIntArrayCage(nMeshes, pointNumberMeshP);
  setIndexVariableCage(pointNumberMeshP, indexPCage, pCagePoint);


  //for vorticity
  indexOmegaXCage = allocate4dIntArrayCage(nMeshes, pointNumberMeshOmegaX);
  setIndexVariableCage(pointNumberMeshOmegaX, indexOmegaXCage, omegaXCagePoint);
 
  indexOmegaYCage = allocate4dIntArrayCage(nMeshes, pointNumberMeshOmegaY);
  setIndexVariableCage(pointNumberMeshOmegaY, indexOmegaYCage, omegaYCagePoint);

  indexOmegaZCage = allocate4dIntArrayCage(nMeshes, pointNumberMeshOmegaZ);
  setIndexVariableCage(pointNumberMeshOmegaZ, indexOmegaZCage, omegaZCagePoint);

  //for velocity
  indexUCage = allocate4dIntArrayCage(nMeshes, pointNumberMeshU);
  setIndexVariableCage(pointNumberMeshU, indexUCage, uCagePoint);

  indexVCage = allocate4dIntArrayCage(nMeshes, pointNumberMeshV);
  setIndexVariableCage(pointNumberMeshV, indexVCage, vCagePoint);

  indexWCage = allocate4dIntArrayCage(nMeshes, pointNumberMeshW);
  setIndexVariableCage(pointNumberMeshW, indexWCage, wCagePoint);


  for (m=1; m<=nCage; m++) {

    // for pressure
    setNVariableCage(nPCage, pointNumberMeshP, pCagePoint);
    indexIPCage = allocate1dArray(nPCage);
    indexJPCage = allocate1dArray(nPCage);
    indexKPCage = allocate1dArray(nPCage);

    // for vorticity
    setNVariableCage(nOmegaXCage, pointNumberMeshOmegaX, omegaXCagePoint);
    indexIOmegaXCage = allocate1dArray(nOmegaXCage);
    indexJOmegaXCage = allocate1dArray(nOmegaXCage);
    indexKOmegaXCage = allocate1dArray(nOmegaXCage);

    setNVariableCage(nOmegaYCage, pointNumberMeshOmegaY, omegaYCagePoint);
    indexIOmegaYCage = allocate1dArray(nOmegaYCage);
    indexJOmegaYCage = allocate1dArray(nOmegaYCage);
    indexKOmegaYCage = allocate1dArray(nOmegaYCage);

    setNVariableCage(nOmegaZCage, pointNumberMeshOmegaZ, omegaZCagePoint);
    indexIOmegaZCage = allocate1dArray(nOmegaZCage);
    indexJOmegaZCage = allocate1dArray(nOmegaZCage);
    indexKOmegaZCage = allocate1dArray(nOmegaZCage);

    // for velocity
    setNVariableCage(nUCage, pointNumberMeshU, uCagePoint);
    indexIUCage = allocate1dArray(nUCage);
    indexJUCage = allocate1dArray(nUCage);
    indexKUCage = allocate1dArray(nUCage);

    setNVariableCage(nVCage, pointNumberMeshV, vCagePoint);
    indexIVCage = allocate1dArray(nVCage);
    indexJVCage = allocate1dArray(nVCage);
    indexKVCage = allocate1dArray(nVCage);

    setNVariableCage(nWCage, pointNumberMeshW, wCagePoint);
    indexIWCage = allocate1dArray(nWCage);
    indexJWCage = allocate1dArray(nWCage);
    indexKWCage = allocate1dArray(nWCage);


    setVector(nPCage,
	      indexPCage,
	      indexIPCage,
	      indexJPCage,
	      indexKPCage,
	      pCage,
	      xPCage,
	      yPCage,   // in the cage class
	      zPCage,   // in the cage class
	      rPCage,
	      thetaPCage,
	      phiPCage,
	      p,
	      xP,
	      pointNumberMeshP,
	      pCagePoint,
	      xParticleCenter,
	      yParticleCenter,
	      zParticleCenter);
   
    setVector(nOmegaXCage,
	      indexOmegaXCage,
	      indexIOmegaXCage,
	      indexJOmegaXCage,
	      indexKOmegaXCage,
	      omegaXCage,
	      xOmegaXCage,
	      yOmegaXCage,   // in the cage class
	      zOmegaXCage,   // in the cage class
	      rOmegaXCage,
	      thetaOmegaXCage,
	      phiOmegaXCage,
	      //outer class
	      omegaX,
	      xOmegaX,
	      pointNumberMeshOmegaX,
	      omegaXCagePoint,
	      xParticleCenter,
	      yParticleCenter,
	      zParticleCenter);

    setVector(nOmegaYCage,
	      indexOmegaYCage,
	      indexIOmegaYCage,
	      indexJOmegaYCage,
	      indexKOmegaYCage,
	      omegaYCage,
	      xOmegaYCage,
	      yOmegaYCage,   // in the cage class
	      zOmegaYCage,   // in the cage class
	      rOmegaYCage,
	      thetaOmegaYCage,
	      phiOmegaYCage,
	      //outer class
	      omegaY,
	      xOmegaY,
	      pointNumberMeshOmegaY,
	      omegaYCagePoint,
	      xParticleCenter,
	      yParticleCenter,
	      zParticleCenter);

    setVector(nOmegaZCage,
	      indexOmegaZCage,
	      indexIOmegaZCage,
	      indexJOmegaZCage,
	      indexKOmegaZCage,
	      omegaZCage,
	      xOmegaZCage,
	      yOmegaZCage,   // in the cage class
	      zOmegaZCage,   // in the cage class
	      rOmegaZCage,
	      thetaOmegaZCage,
	      phiOmegaZCage,
	      //outer class
	      omegaZ,
	      xOmegaZ,
	      pointNumberMeshOmegaZ,
	      omegaZCagePoint,
	      xParticleCenter,
	      yParticleCenter,
	      zParticleCenter);

    setVector(nUCage,
	      indexUCage,
	      indexIUCage,
	      indexJUCage,
	      indexKUCage,
	      uCage,
	      xUCage,
	      yUCage,   // in the cage class
	      zUCage,   // in the cage class
	      rUCage,
	      thetaUCage,
	      phiUCage,
	      //outer class
	      u,
	      xU,
	      pointNumberMeshU,
	      uCagePoint,
	      xParticleCenter,
	      yParticleCenter,
	      zParticleCenter);

    setVector(nVCage,
	      indexVCage,
	      indexIVCage,
	      indexJVCage,
	      indexKVCage,
	      vCage,
	      xVCage,
	      yVCage,   // in the cage class
	      zVCage,   // in the cage class
	      rVCage,
	      thetaVCage,
	      phiVCage,
	      //outer class
	      v,
	      xV,
	      pointNumberMeshV,
	      vCagePoint,
	      xParticleCenter,
	      yParticleCenter,
	      zParticleCenter);

    setVector(nWCage,
	      indexWCage,
	      indexIWCage,
	      indexJWCage,
	      indexKWCage,
	      wCage,
	      xWCage,
	      yWCage,   // in the cage class
	      zWCage,   // in the cage class
	      rWCage,
	      thetaWCage,
	      phiWCage,
	      //outer class
	      w,
	      xW,
	      pointNumberMeshW,
	      wCagePoint,
	      xParticleCenter,
	      yParticleCenter,
	      zParticleCenter);


    // obtain analytical velocity   
    analyticalSol solver;
    solver.solve(pCage,
		 rPCage,
		 thetaPCage,
		 phiPCage,
		 omegaXCage,
		 rOmegaXCage,
		 thetaOmegaXCage,
		 phiOmegaXCage,
		 omegaYCage,
		 rOmegaYCage,
		 thetaOmegaYCage,
		 phiOmegaYCage,
		 omegaZCage,
		 rOmegaZCage,
		 thetaOmegaZCage,
		 phiOmegaZCage,
		 uCage,
		 rUCage,
		 thetaUCage,
		 phiUCage,
		 vCage,
		 rVCage,
		 thetaVCage,
		 phiVCage,
		 wCage,
		 rWCage,
		 thetaWCage,
		 phiWCage,
		 Nc,
		 Re,
		 solvedABCoeffOneParticle,
		 UFixedParticles,
		 SFixedParticles,
		 VFixedParticles,
		 fixedParticles,
		 rotatingParticles,
		 m,
		 angVel,
		 angVelAcc,
		 torqueOld,
		 torqueNew); 

    for (i=0; i<solvedABCoeffOneParticle.size(); i++) {
      solvedABCoeffAllParticleNew[m][i] = solvedABCoeffOneParticle[i];
    }

    //  cout <<"         m = " << m << ", "
    // << "Nondimensional drag = "
    // << (solvedABCoeffAllParticleNew[m][4] 
    //   + 6.0 * solvedABCoeffAllParticleNew[m][6]) * pi / Re /Re
    // <<endl;

    // Cd = Drag/ (0.5 \rho U^2 pi a^2)
    //where U is the inflow velocity
    // a is the radius
    /*
  cout <<"         m = " << m << ", "
       << "Cd = "
       << (solvedABCoeffAllParticleNew[m][4] 
	   + 6.0 * solvedABCoeffAllParticleNew[m][6]) * 2 / Re /Re
       <<endl;
    */
  
    //update velocity
   
    for (i=0; i<nUCage; i++) { 
      //      cout << uCage[i]<<"  " << xUCage[i]<<"  "<< yUCage[i]<<"  "<< rUCage[i]<<"  "<< thetaUCage[i]<<"  "<<endl;
      //      cout << uCage[i]<<"  "<<endl;
      u[m][ indexIUCage[i] ][ indexJUCage[i] ][ indexKUCage[i] ] = uCage[i];
    }
    //            cout <<endl;
    //            cout << "=======  u  =========="<<endl;
    
    for (i=0; i<nVCage; i++) {  
      //      cout << vCage[i] <<"  ";
      v[m][ indexIVCage[i] ][ indexJVCage[i] ][ indexKVCage[i] ] = vCage[i];
    }
    //    cout <<endl;
    //    cout << "======== v ========="<<endl;

    for (i=0; i<nWCage; i++) {  
      //      cout << wCage[i] <<"  ";
      w[m][ indexIWCage[i] ][ indexJWCage[i] ][ indexKWCage[i] ] = wCage[i];
    }
    //    cout <<endl;
    //    cout << "======== w ========="<<endl;
    
    
    delete1dArray(indexIPCage);
    delete1dArray(indexJPCage);
    delete1dArray(indexKPCage);
    delete1dArray(indexIOmegaXCage);
    delete1dArray(indexJOmegaXCage);
    delete1dArray(indexKOmegaXCage);
    delete1dArray(indexIOmegaYCage);
    delete1dArray(indexJOmegaYCage);
    delete1dArray(indexKOmegaYCage);
    delete1dArray(indexIOmegaZCage);
    delete1dArray(indexJOmegaZCage);
    delete1dArray(indexKOmegaZCage);
    delete1dArray(indexIUCage);
    delete1dArray(indexJUCage);
    delete1dArray(indexKUCage);
    delete1dArray(indexIVCage);
    delete1dArray(indexJVCage);
    delete1dArray(indexKVCage);
    delete1dArray(indexIWCage);
    delete1dArray(indexJWCage);
    delete1dArray(indexKWCage);
    
    
  }

    delete4dArrayCage(indexPCage, nMeshes, pointNumberMeshP);
    delete4dArrayCage(indexOmegaXCage, nMeshes, pointNumberMeshOmegaX);
    delete4dArrayCage(indexOmegaYCage, nMeshes, pointNumberMeshOmegaY);
    delete4dArrayCage(indexOmegaZCage, nMeshes, pointNumberMeshOmegaZ);
    delete4dArrayCage(indexUCage, nMeshes, pointNumberMeshU);
    delete4dArrayCage(indexVCage, nMeshes, pointNumberMeshV);
    delete4dArrayCage(indexWCage, nMeshes, pointNumberMeshW);
  
}

void cage::setIndexVariableCage(int ** pointNumberMeshVariable,
				int **** indexVariableCage,
				int **** variableCagePoint){

  for (m=1; m<=nCage; m++) {
    int counter = -1; // make the first point with index 0, consistent to c++ convention
    for (k=0; k<pointNumberMeshVariable[m][2]; k++){
      for (j=0; j<pointNumberMeshVariable[m][1]; j++){
	for (i=0; i<pointNumberMeshVariable[m][0]; i++){
	  indexVariableCage[m][i][j][k] = counter + variableCagePoint[m][i][j][k];
	  counter = indexVariableCage[m][i][j][k];
	}    
      }
    }  
  }

}

void cage::setNVariableCage(int &nVariableCage,
			    int ** pointNumberMeshVariable,
			    int **** variableCagePoint){
  nVariableCage = 0;
  
  for (k=0; k<pointNumberMeshVariable[m][2]; k++){
    for (j=0; j<pointNumberMeshVariable[m][1]; j++){
      for (i=0; i<pointNumberMeshVariable[m][0]; i++){
	nVariableCage += variableCagePoint[m][i][j][k];
      }    
    }
  }

}

void cage::setVector(int nVariableCage,
		     int **** indexVariableCage,
		     int * indexIVariableCage,
		     int * indexJVariableCage,
		     int * indexKVariableCage,
		     vector<real> &variableCage,
		     vector<real> &xVariableCage,
		     vector<real> &yVariableCage,   // in the cage class
		     vector<real> &zVariableCage,   // in the cage class
		     vector<real> &rVariableCage,
		     vector<real> &thetaVariableCage,
		     vector<real> &phiVariableCage,
		     //outer class 
		     real **** variable,
		     real *** xVariable,
		     int ** pointNumberMeshVariable,
		     int **** variableCagePoint,
		     real * xParticleCenter,
		     real * yParticleCenter,
		     real * zParticleCenter){

  // allocate memory  
  variableCage.assign(nVariableCage, 0.0);
  xVariableCage.assign(nVariableCage, 0.0);
  yVariableCage.assign(nVariableCage, 0.0);
  zVariableCage.assign(nVariableCage, 0.0);
  rVariableCage.assign(nVariableCage, 0.0);
  thetaVariableCage.assign(nVariableCage, 0.0);
  phiVariableCage.assign(nVariableCage, 0.0);
  
  for (k=pointNumberMeshVariable[m][2]-1; k>=0; k--) {
    for (j=pointNumberMeshVariable[m][1]-1; j>=0; j--) {
      for (i=pointNumberMeshVariable[m][0]-1; i>=0; i--) {
	
	// note that for the starting points
	// indexVariableCage is -1,
	// which cannot be used as an index
	// so have to use the if statement
	if (variableCagePoint[m][i][j][k] == 1) {
	  int iC = indexVariableCage[m][i][j][k];
	  indexIVariableCage[iC] = i;
	  indexJVariableCage[iC] = j;
	  indexKVariableCage[iC] = k;
	  variableCage[iC] = variable[m][i][j][k];
	  xVariableCage[iC] = xVariable[m][0][i];
	  yVariableCage[iC] = xVariable[m][1][j];
	  zVariableCage[iC] = xVariable[m][2][k];
	}
	
      } 
    } 
  }

  // from cartesian to spherical  
  for (i=0; i<nVariableCage; i++){
    rVariableCage[i] = sqrt( pow(xVariableCage[i] - xParticleCenter[m] , 2.0) 
			     + pow(yVariableCage[i] - yParticleCenter[m] , 2.0)
			     + pow(zVariableCage[i] - zParticleCenter[m] , 2.0) );
    thetaVariableCage[i] = acos( (zVariableCage[i] - zParticleCenter[m])
			       / rVariableCage[i] );
    phiVariableCage[i] = atan2(yVariableCage[i] - yParticleCenter[m],  
				 xVariableCage[i] - xParticleCenter[m] );
  }
  
  
}

int * cage::allocate1dArray(int size){
  int *x = new int[size];
  return x;
}

void cage::delete1dArray(int * x){
  delete [] x;
}


int **** cage::allocate4dIntArrayCage(int nMeshes, int **pointNumberMesh){

  int ****x = new int ***[nMeshes];

  //don't allocate memory for m=0, since never needed for cage
  for (m=1; m<nMeshes; m++) { 
    x[m] = new int **[pointNumberMesh[m][0]];
    for (i=0; i<pointNumberMesh[m][0]; i++) {
      x[m][i] = new int*[pointNumberMesh[m][1]];
      for (j=0; j<pointNumberMesh[m][1]; j++) {
	x[m][i][j] = new int[pointNumberMesh[m][2]];
      }
    }
  }
  return x;
}
void cage::delete4dArrayCage(int ****x, int nMeshes, int **pointNumberMesh){
  // note that no memory is allocated for m=0 
  for (m=1; m<nMeshes; m++) {
    for (i=0; i<pointNumberMesh[m][0]; i++) {
      for (j=0; j<pointNumberMesh[m][1]; j++) {
	delete [] x[m][i][j];
      }
      delete [] x[m][i];
    }
    delete [] x[m];
  }
  delete [] x;
}

cage::~cage(){
}
