#include "cageInit.h"

// note that this file is not related with m=0,
// the base mesh

cageInit::cageInit(){
}

void cageInit::solve(int numberOfCage,
		     int cellNumberPerPatch,
		     int dimension,
		     // for u
		     real ****u,
		     int **pointNumberMeshU,
		     int  ****uCagePoint,
		     // for v
		     real ****v,
		     int **pointNumberMeshV,
		     int  ****vCagePoint,
		     // for w
		     real ****w,
		     int **pointNumberMeshW,
		     int  ****wCagePoint){

  nCage = numberOfCage;
  nMeshes = nCage + 1;  //need to modify for patch merging
  nCellPerPatch = cellNumberPerPatch;
  dim = dimension;


  indexUCage = allocate4dIntArrayCage(nMeshes, pointNumberMeshU); 
  for (m=1; m<=nCage; m++) {
    // make the first point in each cage
    // with index 0, consistent to c++ convention
    int counter = -1; 
    for (k=0; k<pointNumberMeshU[m][2]; k++){
      for (j=0; j<pointNumberMeshU[m][1]; j++){
	for (i=0; i<pointNumberMeshU[m][0]; i++){
	  indexUCage[m][i][j][k] = counter + uCagePoint[m][i][j][k];
	  counter = indexUCage[m][i][j][k];
	}    
      }
    }

  }  

  indexVCage = allocate4dIntArrayCage(nMeshes, pointNumberMeshV);
  for (m=1; m<=nCage; m++) {
    int counter = -1; // make the first point with index 0, consistent to c++ convention
    for (k=0; k<pointNumberMeshV[m][2]; k++){
      for (j=0; j<pointNumberMeshV[m][1]; j++){
	for (i=0; i<pointNumberMeshV[m][0]; i++){
	  indexVCage[m][i][j][k] = counter + vCagePoint[m][i][j][k];
	  counter = indexVCage[m][i][j][k];
	}    
      }
    }
  }  

  indexWCage = allocate4dIntArrayCage(nMeshes, pointNumberMeshW);
  for (m=1; m<=nCage; m++) {
    int counter = -1; // make the first point with index 0, consistent to c++ convention
    for (k=0; k<pointNumberMeshW[m][2]; k++){
      for (j=0; j<pointNumberMeshW[m][1]; j++){
	for (i=0; i<pointNumberMeshW[m][0]; i++){
	  indexWCage[m][i][j][k] = counter + wCagePoint[m][i][j][k];
	  counter = indexWCage[m][i][j][k];
	}    
      }
    }
  }  


  for (m=1; m<=nCage; m++) {

    setNVariableCage(nUCage,
		     pointNumberMeshU,
		     uCagePoint);
    indexIUCage = allocate1dArray(nUCage);
    indexJUCage = allocate1dArray(nUCage);
    indexKUCage = allocate1dArray(nUCage);

    setNVariableCage(nVCage,
		     pointNumberMeshV,
		     vCagePoint);
    indexIVCage = allocate1dArray(nVCage);
    indexJVCage = allocate1dArray(nVCage);
    indexKVCage = allocate1dArray(nVCage);

    setNVariableCage(nWCage,
		     pointNumberMeshW,
		     wCagePoint);
    indexIWCage = allocate1dArray(nWCage);
    indexJWCage = allocate1dArray(nWCage);
    indexKWCage = allocate1dArray(nWCage);

   
    setIndex(indexUCage,
	     indexIUCage,
	     indexJUCage,
	     indexKUCage,
	     //outer class
	     pointNumberMeshU,
	     uCagePoint);

    setIndex(indexVCage,
	     indexIVCage,
	     indexJVCage,
	     indexKVCage,
	     //outer class
	     pointNumberMeshV,
	     vCagePoint);

    setIndex(indexWCage,
	     indexIWCage,
	     indexJWCage,
	     indexKWCage,
	     //outer class
	     pointNumberMeshW,
	     wCagePoint);

    // set velocity at all velocity cage points
    // to zero, 
    // in order to have a somewhat physical
    // initial velocity distribution
   
    for (i=0; i<nUCage; i++) { 
      u[m][ indexIUCage[i] ][ indexJUCage[i] ][ indexKUCage[i] ] = 0.0; 
    }
    
    for (i=0; i<nVCage; i++) {  
      v[m][ indexIVCage[i] ][ indexJVCage[i] ][ indexKVCage[i] ] = 0.0;
    }    

    for (i=0; i<nWCage; i++) {  
      w[m][ indexIWCage[i] ][ indexJWCage[i] ][ indexKWCage[i] ] = 0.0;
    }    
    
    delete1dArray(indexIUCage);
    delete1dArray(indexJUCage);
    delete1dArray(indexKUCage);
    delete1dArray(indexIVCage);
    delete1dArray(indexJVCage);
    delete1dArray(indexKVCage);
    delete1dArray(indexIWCage);
    delete1dArray(indexJWCage);
    delete1dArray(indexKWCage);
    
    
  }

    delete4dArrayCage(indexUCage, nMeshes, pointNumberMeshU);
    delete4dArrayCage(indexVCage, nMeshes, pointNumberMeshV);
    delete4dArrayCage(indexWCage, nMeshes, pointNumberMeshW);
  
}

void cageInit::setNVariableCage(int &nVariableCage,
			    int ** pointNumberMeshVariable,
			    int **** variableCagePoint){
  nVariableCage = 0;
  
  for (k=0; k<pointNumberMeshVariable[m][2]; k++){
    for (j=0; j<pointNumberMeshVariable[m][1]; j++){
      for (i=0; i<pointNumberMeshVariable[m][0]; i++){
	nVariableCage += variableCagePoint[m][i][j][k];
      }    
    }
  }

}

void cageInit::setIndex(int **** indexVariableCage,
			int * indexIVariableCage,
			int * indexJVariableCage,
			int * indexKVariableCage,
			//outer class 
			int ** pointNumberMeshVariable,
			int **** variableCagePoint){
  
  for (k=pointNumberMeshVariable[m][2]-1; k>=0; k--) {
    for (j=pointNumberMeshVariable[m][1]-1; j>=0; j--) {
      for (i=pointNumberMeshVariable[m][0]-1; i>=0; i--) {
	
	// note that for the starting points
	// indexVariableCage is -1,
	// which cannot be used as an index
	// so have to use the if statement
	if (variableCagePoint[m][i][j][k] == 1) {
	  int iC = indexVariableCage[m][i][j][k];
	  indexIVariableCage[iC] = i;
	  indexJVariableCage[iC] = j;
	  indexKVariableCage[iC] = k;
	}
	
      } 
    } 
  }
  
  
}

int * cageInit::allocate1dArray(int size){
  int *x = new int[size];
  return x;
}

void cageInit::delete1dArray(int * x){
  delete [] x;
}


int **** cageInit::allocate4dIntArrayCage(int nMeshes, int **pointNumberMesh){

  int ****x = new int ***[nMeshes];

  //don't allocate memory for m=0, since never needed for cage
  for (m=1; m<nMeshes; m++) {
    x[m] = new int **[pointNumberMesh[m][0]];
    for (i=0; i<pointNumberMesh[m][0]; i++) {
      x[m][i] = new int*[pointNumberMesh[m][1]];
      for (j=0; j<pointNumberMesh[m][1]; j++) {
	x[m][i][j] = new int[pointNumberMesh[m][2]];
      }
    }
  }
  return x;
}
void cageInit::delete4dArrayCage(int ****x, int nMeshes, int **pointNumberMesh){

  // note that no memory is allocated for m=0 
  for (m=1; m<nMeshes; m++) {
    for (i=0; i<pointNumberMesh[m][0]; i++) {
      for (j=0; j<pointNumberMesh[m][1]; j++) {
	delete [] x[m][i][j];
      }
      delete [] x[m][i];
    }
    delete [] x[m];
  }
  delete [] x;
}


cageInit::~cageInit(){
}
