#ifndef CAGEINIT_H
#define CAGEINIT_H

/** \file cageInit.cpp
    \brief set velocity for cage points at the initial stage

    This class sets the velocity for cage points at the initial stage. They may be zero or some values from experience or knowledge.

*/

#ifdef SINGLE
#define real float
#else
#define real double
#endif


#include <iostream>
#include <string>
#include <cmath>
#include <vector>
using namespace std;

/** 
    \brief set velocity for cage points at the initial stage

    This class sets the velocity for cage points at the initial stage. They may be zero or some values from experience or knowledge.

*/

class cageInit {
  
 public:
  /**
     constructor
   */
  cageInit();

/** 
    \brief set velocity for cage points at the initial stage

    Sets the velocity for cage points at the initial stage. They may be zero or some values from experience or knowledge. This only makes the initial conditions more physical.

*/
  void solve(int numberOfCage, /**< number of cages */
	     int cellNumberPerPatch, /**<  cell numbers per patch surrounding the sphere*/
	     int dimension, /**<  dimension */
	     // for u
	     real ****u, /**< u */
	     int **pointNumberMeshU, /**< mesh points for u */
	     int  ****uCagePoint, /**< cage points for u */
	     // for v
	     real ****v, /**< v */
	     int **pointNumberMeshV, /**< mesh points for v */
	     int  ****vCagePoint, /**< cage points for v */
	     // for w
	     real ****w, /**< w */
	     int **pointNumberMeshW, /**< mesh points for w */
	     int  ****wCagePoint/**< cage points for w */
	     );

  /**
     destructor
   */
  ~cageInit();
  
 private: 

  void setNVariableCage(int &nVariableCage,
			int ** pointNumberMeshVariable,
			int **** variableCagePoint);
  void setIndex(int **** indexVariableCage,
		int * indexIVariableCage,
		int * indexJVariableCage,
		int * indexKVariableCage,
		//outer class 
		int ** pointNumberMeshVariable,
		int **** variableCagePoint);

  int * allocate1dArray(int size);
  void delete1dArray(int * x);

  int **** allocate4dIntArrayCage(int nMeshes, int **pointNumberMesh);
  void delete4dArrayCage(int ****x, int nMeshes, int **pointNumberMesh);

  int i, j, k, m; // dummy index
 
  // number of pressure points and vortivity points
  int nCage;
  int nMeshes;
  int nCellPerPatch;
  int dim;

  int **** indexUCage;
  int * indexIUCage;
  int * indexJUCage;
  int * indexKUCage;
  int nUCage;

  int **** indexVCage;
  int * indexIVCage;
  int * indexJVCage;
  int * indexKVCage;
  int nVCage;

  int **** indexWCage;
  int * indexIWCage;
  int * indexJWCage;
  int * indexKWCage;
  int nWCage;


};

#endif
