#include "divUStar.h"


divUStar::divUStar(){
}

void divUStar::initialize(int numberOfPatches,
		      real dt,
		      int cellNumberPerPatch, 
		      int ** StartIndex, 
		      real ****fp, //force for pressure poisson equation
		      int ** pointNumberMeshP, 
		      real *h,
		      real ***xP,
		      real ****uAux,
		      real ****vAux,
		      real ****wAux){

  nPatch = numberOfPatches;
  nMeshes = numberOfPatches + 1;
  nCellPerPatch = cellNumberPerPatch;


  for (k=0; k<pointNumberMeshP[0][2]; k++) {
    for (j=0; j<pointNumberMeshP[0][1]; j++) {
      for (i=0; i<pointNumberMeshP[0][0]; i++) {
	fp[0][i][j][k] = ( ( uAux[0][i+1][j+1][k+1] - uAux[0][i][j+1][k+1] ) / h[0]
			   + ( vAux[0][i+1][j+1][k+1] - vAux[0][i+1][j][k+1] ) / h[0] 
			   + ( wAux[0][i+1][j+1][k+1] - wAux[0][i+1][j+1][k] ) / h[0])
	  / dt;
      }
    }
  }
  

  for (m=1; m<=nPatch; m++) {
    for (k=0; k<pointNumberMeshP[m][2]; k++) {
      for (j=0; j<pointNumberMeshP[m][1]; j++) {
	for (i=0; i<pointNumberMeshP[m][0]; i++) {
	  fp[m][i][j][k] = ( ( uAux[m][i+1][j][k] - uAux[m][i][j][k] ) / h[m]
			     + ( vAux[m][i][j+1][k] - vAux[m][i][j][k] ) / h[m]
			     + ( wAux[m][i][j][k+1] - wAux[m][i][j][k] ) / h[m] )
	    / dt;
	}
      }
    }
  }

  /*
  for (i=0; i<pointNumberMeshP[0][0]; i++) {
    fp[0][i][pointNumberMeshP[0][1]-1] = 0.0;
  }
  
  for (i=0; i<pointNumberMeshP[0][0]; i++) {
    fp[0][i][0] = 0.0 ;
  }
  
  for (j=0; j<pointNumberMeshP[0][1]; j++) {
    //      for (i=0; i<pointNumberMeshP[m][0]; i++) {
    fp[0][0][j] = 0.0 ;
  }
  
  for (j=0; j<pointNumberMeshP[0][1]; j++) {
    fp[0][pointNumberMeshP[0][0]-1][j] = 0.0;
  }
  */


  /*
  for (j=0; j<pointNumberMeshP[0][1]; j++) {
    for (i=0; i<pointNumberMeshP[0][0]; i++) {
      fp[0][i][j] = xP[0][i][j] + yP[0][i][j] ;
    }
  }


  for (m=1; m<=nPatch; m++) {
    for (j=0; j<pointNumberMeshP[m][1]; j++) {
      for (i=0; i<pointNumberMeshP[m][0]; i++) {
	fp[m][i][j] = xP[m][i][j] + yP[m][i][j];
      }
    }
  }

  
  for (m=1; m<=nPatch; m++) {
    for (j=0; j<pointNumberMeshP[m][1]; j++) {
      for (i=0; i<pointNumberMeshP[m][0]; i++) {
	cout << fp[m][i][j] <<" ";
      }
      cout <<endl;
    }
    cout <<"========================="<<endl;
  }
  */  

  //
  //points in coarse mesh and inside partilces
  //

  for (m=1; m<=nPatch; m++) {


    //west surface
    i = StartIndex[m][0];
    for (k=StartIndex[m][2]; k<=StartIndex[m][2]+cellNumberPerPatch-1; k++){
      for (j=StartIndex[m][1]; j<=StartIndex[m][1]+cellNumberPerPatch-1; j++){
	
	// left bottom corner index
	iFine = 2;
	jFine = 2 + 2 * (j - StartIndex[m][1]);
	kFine = 2 + 2 * (k - StartIndex[m][2]);

	coarseGhostPoint(fp);		
      }
    }


    //east surface
    i = StartIndex[m][0] + cellNumberPerPatch-1;
    for (k=StartIndex[m][2]; k<=StartIndex[m][2]+cellNumberPerPatch-1; k++){
      for (j=StartIndex[m][1]; j<=StartIndex[m][1]+cellNumberPerPatch-1; j++){

	// left bottom corner index
	iFine = 2 + 2 * (cellNumberPerPatch - 1);
	jFine = 2 + 2 * (j - StartIndex[m][1]);
	kFine = 2 + 2 * (k - StartIndex[m][2]);
	
	coarseGhostPoint(fp);		
      }
    }

    //south surface
    j = StartIndex[m][1];
    for (k=StartIndex[m][2]; k<=StartIndex[m][2]+nCellPerPatch-1; k++){
      for (i=StartIndex[m][0]; i<=StartIndex[m][0]+nCellPerPatch-1; i++){
	
	// left bottom corner index
	iFine = 2 + 2 * (i - StartIndex[m][0]);
	jFine = 2;
	kFine = 2 + 2 * (k - StartIndex[m][2]);
	
	coarseGhostPoint(fp);	
      }
    }
    
    //north surface
    j = StartIndex[m][1] + cellNumberPerPatch - 1;
    for (k=StartIndex[m][2]; k<=StartIndex[m][2]+cellNumberPerPatch-1; k++){
      for (i=StartIndex[m][0]; i<=StartIndex[m][0]+cellNumberPerPatch-1; i++){
	
	// left bottom corner index
	iFine = 2 + 2 * (i - StartIndex[m][0]);
	jFine = 2 + 2 * (cellNumberPerPatch - 1);
	kFine = 2 + 2 * (k - StartIndex[m][2]);

	coarseGhostPoint(fp);	
      }
    }

    //bottom surface
    k = StartIndex[m][2];
    for (j=StartIndex[m][1]; j<=StartIndex[m][1]+nCellPerPatch-1; j++){
      for (i=StartIndex[m][0]; i<=StartIndex[m][0]+nCellPerPatch-1; i++){

	// left bottom corner index
	iFine = 2 + 2 * (i - StartIndex[m][0]);
	jFine = 2 + 2 * (j - StartIndex[m][1]);
	kFine = 2;

	coarseGhostPoint(fp);	
      }
    }

    //top surface
    k = StartIndex[m][2] + cellNumberPerPatch - 1;
    for (j=StartIndex[m][1]; j<=StartIndex[m][1]+nCellPerPatch-1; j++){
      for (i=StartIndex[m][0]; i<=StartIndex[m][0]+nCellPerPatch-1; i++){

	// left bottom corner index
	iFine = 2 + 2 * (i - StartIndex[m][0]);
	jFine = 2 + 2 * (j - StartIndex[m][1]);
	kFine = 2 + 2 * (cellNumberPerPatch - 1);

	coarseGhostPoint(fp);	
      }
    }
    
  }


  //
  //points in fine mesh
  //

  for (m=1; m<=nPatch; m++) {

    // note: this part is copied from pressure.cpp
    //       only delete the argument h in fineGhostPoint

    //////////////////////////////////////////////
    //
    //  x direction
    //
    /////////////////////////////////////////////

    //west:
    iFine = 0;
    //odd j, //odd k
    for (kFine=0; kFine<=pointNumberMeshP[m][2]-1; kFine=kFine+2){
      for (jFine=0; jFine<=pointNumberMeshP[m][1]-1; jFine=jFine+2){
      
	// west south bottom corner index
	i = StartIndex[m][0]-2;
	j = StartIndex[m][1]-2 + jFine / 2;
	k = StartIndex[m][2]-2 + kFine / 2;

	fineGhostPoint(0, // 0 for odd; 1 for even in x direction
		       0, // 0 for odd; 1 for even in y direction
		       0, // 0 for odd; 1 for even in z direction
		       fp);	
      }
    }

    //west:
    iFine = 0;
    //odd j,even k
    for (kFine=1; kFine<=pointNumberMeshP[m][2]-1; kFine=kFine+2){
      for (jFine=0; jFine<=pointNumberMeshP[m][1]-1; jFine=jFine+2){
	
	// left bottom corner index
	i = StartIndex[m][0]-2;
	j = StartIndex[m][1]-2 + jFine / 2;
	k = StartIndex[m][2]-1 + (kFine-1) / 2;
	
	fineGhostPoint(0, // 0 for odd; 1 for even in x direction
		       0, // 0 for odd; 1 for even in y direction
		       1, // 0 for odd; 1 for even in z direction
		       fp);	
      }
      
    }

    //west:
    iFine = 0;
    //even j, odd k
    for (kFine=0; kFine<=pointNumberMeshP[m][2]-1; kFine=kFine+2){
      for (jFine=1; jFine<=pointNumberMeshP[m][1]-1; jFine=jFine+2){
	//pay special attention to corner
	
	// left bottom corner index
	i = StartIndex[m][0]-2;
	j = StartIndex[m][1]-1 + (jFine-1) / 2;
	k = StartIndex[m][2]-2 + kFine / 2;

  	fineGhostPoint(0, // 0 for odd; 1 for even in x direction
		       1, // 0 for odd; 1 for even in y direction
		       0, // 0 for odd; 1 for even in z direction
		       fp);	
      }
    }

    //west:
    iFine = 0;
    //even j, even k
    for (kFine=1; kFine<=pointNumberMeshP[m][2]-1; kFine=kFine+2){
      for (jFine=1; jFine<=pointNumberMeshP[m][1]-1; jFine=jFine+2){
	
	// left bottom corner index
	i = StartIndex[m][0]-2;
	j = StartIndex[m][1]-1 + (jFine-1) / 2;
	k = StartIndex[m][2]-1 + (kFine-1) / 2;

	fineGhostPoint(0, // 0 for odd; 1 for even in x direction
		       1, // 0 for odd; 1 for even in y direction
		       1, // 0 for odd; 1 for even in z direction
		       fp);	
      }
      
    }


    //east:
    iFine = pointNumberMeshP[m][0]-1;

    // j odd, k odd
    for (kFine=0; kFine<=pointNumberMeshP[m][2]-1; kFine=kFine+2){
      for (jFine=0; jFine<=pointNumberMeshP[m][1]-1; jFine=jFine+2){

	// left bottom corner index
	i = StartIndex[m][0] + cellNumberPerPatch;
	j = StartIndex[m][1]-2 + jFine / 2;
	k = StartIndex[m][2]-2 + kFine / 2;

	fineGhostPoint(1, // 0 for odd; 1 for even in x direction
		       0, // 0 for odd; 1 for even in y direction
		       0, // 0 for odd; 1 for even in z direction
		       fp);	
      }
    }

    //east:
    iFine = pointNumberMeshP[m][0]-1;
    // j odd, k even
    for (kFine=1; kFine<=pointNumberMeshP[m][2]-1; kFine=kFine+2){
      for (jFine=0; jFine<=pointNumberMeshP[m][1]-1; jFine=jFine+2){
	
	// left bottom corner index
	i = StartIndex[m][0] + cellNumberPerPatch;
	j = StartIndex[m][1]-2 + jFine / 2;
	k = StartIndex[m][2]-1 + (kFine-1) / 2;

 	fineGhostPoint(1, // 0 for odd; 1 for even in x direction
		       0, // 0 for odd; 1 for even in y direction
		       1, // 0 for odd; 1 for even in z direction
		       fp);	
     }
    }


    //east:
    iFine = pointNumberMeshP[m][0]-1;

    // j even, k odd
    for (kFine=0; kFine<=pointNumberMeshP[m][2]-1; kFine=kFine+2){
      for (jFine=1; jFine<=pointNumberMeshP[m][1]-1; jFine=jFine+2){

	// left bottom corner index
	i = StartIndex[m][0] + cellNumberPerPatch;
	j = StartIndex[m][1]-1 + (jFine-1) / 2;
	k = StartIndex[m][2]-2 + kFine / 2;

	fineGhostPoint(1, // 0 for odd; 1 for even in x direction
		       1, // 0 for odd; 1 for even in y direction
		       0, // 0 for odd; 1 for even in z direction
		       fp);	
      }
    }

    //east:
    iFine = pointNumberMeshP[m][0]-1;
    // j even, k even
    for (kFine=1; kFine<=pointNumberMeshP[m][2]-1; kFine=kFine+2){
      for (jFine=1; jFine<=pointNumberMeshP[m][1]-1; jFine=jFine+2){
	
	// left bottom corner index
	i = StartIndex[m][0] + cellNumberPerPatch;
	j = StartIndex[m][1]-1 + (jFine-1) / 2;
	k = StartIndex[m][2]-1 + (kFine-1) / 2;

	fineGhostPoint(1, // 0 for odd; 1 for even in x direction
		       1, // 0 for odd; 1 for even in y direction
		       1, // 0 for odd; 1 for even in z direction
		       fp);	
      }
      
    }



    //////////////////////////////////////////////
    //
    //  y direction
    //
    /////////////////////////////////////////////

    //pay special attention to those shared by surfaces


    //south:
    jFine = 0;
    //odd i, //odd k
    for (kFine=0; kFine<=pointNumberMeshP[m][2]-1; kFine=kFine+2){
      for (iFine=0; iFine<=pointNumberMeshP[m][0]-1; iFine=iFine+2){
     
	// west south bottom corner index
	i = StartIndex[m][0]-2 + iFine / 2;
	j = StartIndex[m][1]-2;
	k = StartIndex[m][2]-2 + kFine / 2;
	
	fineGhostPoint(0, // 0 for odd; 1 for even in x direction
		       0, // 0 for odd; 1 for even in y direction
		       0, // 0 for odd; 1 for even in z direction
		       fp);	
      }
    }

    //south:
    jFine = 0;
    //odd i,even k
    for (kFine=1; kFine<=pointNumberMeshP[m][2]-1; kFine=kFine+2){
      for (iFine=0; iFine<=pointNumberMeshP[m][0]-1; iFine=iFine+2){
	
	// left bottom corner index
	i = StartIndex[m][0]-2 + iFine / 2;
	j = StartIndex[m][1]-2;
	k = StartIndex[m][2]-1 + (kFine-1) / 2;
	
	fineGhostPoint(0, // 0 for odd; 1 for even in x direction
		       0, // 0 for odd; 1 for even in y direction
		       1, // 0 for odd; 1 for even in z direction
		       fp);	
      }
    }

    //south:
    jFine = 0;
    //even i, odd k
    for (kFine=0; kFine<=pointNumberMeshP[m][2]-1; kFine=kFine+2){
      for (iFine=1; iFine<=pointNumberMeshP[m][0]-1; iFine=iFine+2){
		
	// left bottom corner index
	i = StartIndex[m][0]-1 + (iFine-1) / 2;
	j = StartIndex[m][1]-2;
	k = StartIndex[m][2]-2 + kFine / 2;

 	fineGhostPoint(1, // 0 for odd; 1 for even in x direction
		       0, // 0 for odd; 1 for even in y direction
		       0, // 0 for odd; 1 for even in z direction
		       fp);	 
      }
    }

    //south:
    jFine = 0;
    //even i, even k
    for (kFine=1; kFine<=pointNumberMeshP[m][2]-1; kFine=kFine+2){
      for (iFine=1; iFine<=pointNumberMeshP[m][0]-1; iFine=iFine+2){
	
	// left bottom corner index
	i = StartIndex[m][0]-1 + (iFine-1) / 2;
	j = StartIndex[m][1]-2;
	k = StartIndex[m][2]-1 + (kFine-1) / 2;

	fineGhostPoint(1, // 0 for odd; 1 for even in x direction
		       0, // 0 for odd; 1 for even in y direction
		       1, // 0 for odd; 1 for even in z direction
		       fp);
      }
    }


    //north:
    jFine = pointNumberMeshP[m][1]-1;

    // i odd, k odd
    for (kFine=0; kFine<=pointNumberMeshP[m][2]-1; kFine=kFine+2){
      for (iFine=0; iFine<=pointNumberMeshP[m][0]-1; iFine=iFine+2){
     //pay special attention to those shared by surfaces

	// left bottom corner index
	i = StartIndex[m][0]-2 + iFine / 2;
	j = StartIndex[m][1] + cellNumberPerPatch;
	k = StartIndex[m][2]-2 + kFine / 2;

	fineGhostPoint(0, // 0 for odd; 1 for even in x direction
		       1, // 0 for odd; 1 for even in y direction
		       0, // 0 for odd; 1 for even in z direction
		       fp); 
      }
    }

    //north: j even
    jFine = pointNumberMeshP[m][1]-1;
    // i odd, k even
    for (kFine=1; kFine<=pointNumberMeshP[m][2]-1; kFine=kFine+2){
      for (iFine=0; iFine<=pointNumberMeshP[m][0]-1; iFine=iFine+2){
	
	// left bottom corner index
	i = StartIndex[m][0]-2 + iFine / 2;
	j = StartIndex[m][1] + cellNumberPerPatch;
	k = StartIndex[m][2]-1 + (kFine-1) / 2;

	fineGhostPoint(0, // 0 for odd; 1 for even in x direction
		       1, // 0 for odd; 1 for even in y direction
		       1, // 0 for odd; 1 for even in z direction
		       fp); 
      }
    }


    //north:
    jFine = pointNumberMeshP[m][1]-1;

    // i even, k odd
    for (kFine=0; kFine<=pointNumberMeshP[m][2]-1; kFine=kFine+2){
      for (iFine=1; iFine<=pointNumberMeshP[m][0]-1; iFine=iFine+2){
     //pay special attention to those shared by surfaces

	// left bottom corner index
	j = StartIndex[m][1] + cellNumberPerPatch;
	i = StartIndex[m][0]-1 + (iFine-1) / 2;
	k = StartIndex[m][2]-2 + kFine / 2;

	fineGhostPoint(1, // 0 for odd; 1 for even in x direction
		       1, // 0 for odd; 1 for even in y direction
		       0, // 0 for odd; 1 for even in z direction
		       fp); 
      }
    }

    //north:
    jFine = pointNumberMeshP[m][1]-1;
    // i even, k even
    for (kFine=1; kFine<=pointNumberMeshP[m][2]-1; kFine=kFine+2){
      for (iFine=1; iFine<=pointNumberMeshP[m][0]-1; iFine=iFine+2){
	
	// left bottom corner index
	i = StartIndex[m][0]-1 + (iFine-1) / 2;
	j = StartIndex[m][1] + cellNumberPerPatch;
	k = StartIndex[m][2]-1 + (kFine-1) / 2;

	fineGhostPoint(1, // 0 for odd; 1 for even in x direction
		       1, // 0 for odd; 1 for even in y direction
		       1, // 0 for odd; 1 for even in z direction
		       fp); 
      }     
    }





    //////////////////////////////////////////////
    //
    //  z direction
    //
    /////////////////////////////////////////////

    //bottom:
   kFine = 0;
    //odd j, //odd k
   for (jFine=0; jFine<=pointNumberMeshP[m][1]-1; jFine=jFine+2){
     for (iFine=0; iFine<=pointNumberMeshP[m][0]-1; iFine=iFine+2){
	
	// west south bottom corner index
	i = StartIndex[m][0]-2 + iFine / 2;
	j = StartIndex[m][1]-2 + jFine / 2;
	k = StartIndex[m][2]-2;

	fineGhostPoint(0, // 0 for odd; 1 for even in x direction
		       0, // 0 for odd; 1 for even in y direction
		       0, // 0 for odd; 1 for even in z direction
		       fp); 
      }
    }

    //bottom:
    kFine = 0;
    //odd j,even i
    for (jFine=0; jFine<=pointNumberMeshP[m][1]-1; jFine=jFine+2){
      for (iFine=1;iFine<=pointNumberMeshP[m][0]-1; iFine=iFine+2){
	
	// left bottom corner index
	i = StartIndex[m][0]-1 + (iFine-1) / 2;
	j = StartIndex[m][1]-2 + jFine / 2;
	k = StartIndex[m][2]-2;
	
	fineGhostPoint(1, // 0 for odd; 1 for even in x direction
		       0, // 0 for odd; 1 for even in y direction
		       0, // 0 for odd; 1 for even in z direction
		       fp); 
      }      
    }


    //bottom:
    kFine = 0;
    //even j, odd i
    for (iFine=0; iFine<=pointNumberMeshP[m][0]-1; iFine=iFine+2){
      for (jFine=1; jFine<=pointNumberMeshP[m][1]-1; jFine=jFine+2){
	//pay special attention to corner
	
	// left bottom corner index
	i = StartIndex[m][0]-2 + iFine / 2;
	j = StartIndex[m][1]-1 + (jFine-1) / 2;
	k = StartIndex[m][2]-2;

	fineGhostPoint(0, // 0 for odd; 1 for even in x direction
		       1, // 0 for odd; 1 for even in y direction
		       0, // 0 for odd; 1 for even in z direction
		       fp); 
      }
    }

    //bottom:
    kFine = 0;
    //even j, even i
    for (iFine=1; iFine<=pointNumberMeshP[m][0]-1; iFine=iFine+2){
      for (jFine=1; jFine<=pointNumberMeshP[m][1]-1; jFine=jFine+2){
	
	// left bottom corner index
	i = StartIndex[m][0]-1 + (iFine-1) / 2;
	j = StartIndex[m][1]-1 + (jFine-1) / 2;
	k = StartIndex[m][2]-2;

	fineGhostPoint(1, // 0 for odd; 1 for even in x direction
		       1, // 0 for odd; 1 for even in y direction
		       0, // 0 for odd; 1 for even in z direction
		       fp); 
      }      
    }


    //top:
    kFine = pointNumberMeshP[m][2]-1;

    // j odd, i odd
    for (iFine=0; iFine<=pointNumberMeshP[m][0]-1; iFine=iFine+2){
      for (jFine=0; jFine<=pointNumberMeshP[m][1]-1; jFine=jFine+2){
	
	// left bottom corner index
	i = StartIndex[m][0]-2 + iFine / 2;
	j = StartIndex[m][1]-2 + jFine / 2;
	k = StartIndex[m][2] + cellNumberPerPatch;

	fineGhostPoint(0, // 0 for odd; 1 for even in x direction
		       0, // 0 for odd; 1 for even in y direction
		       1, // 0 for odd; 1 for even in z direction
		       fp); 
      }
    }

    //top:
    kFine = pointNumberMeshP[m][2]-1;
    // j odd, i even
    for (iFine=1; iFine<=pointNumberMeshP[m][0]-1; iFine=iFine+2){
      for (jFine=0; jFine<=pointNumberMeshP[m][1]-1; jFine=jFine+2){

	// left bottom corner index
	i = StartIndex[m][0]-1 + (iFine-1) / 2;
	j = StartIndex[m][1]-2 + jFine / 2;
	k = StartIndex[m][2] + cellNumberPerPatch;

	fineGhostPoint(1, // 0 for odd; 1 for even in x direction
		       0, // 0 for odd; 1 for even in y direction
		       1, // 0 for odd; 1 for even in z direction
		       fp); 
      }
    }


    //top:
    kFine = pointNumberMeshP[m][2]-1;

    // j even, i odd
    for (iFine=0; iFine<=pointNumberMeshP[m][0]-1; iFine=iFine+2){
      for (jFine=1; jFine<=pointNumberMeshP[m][1]-1; jFine=jFine+2){

	// left bottom corner index
	k = StartIndex[m][2] + cellNumberPerPatch;
	j = StartIndex[m][1]-1 + (jFine-1) / 2;
	i = StartIndex[m][0]-2 + iFine / 2;

	fineGhostPoint(0, // 0 for odd; 1 for even in x direction
		       1, // 0 for odd; 1 for even in y direction
		       1, // 0 for odd; 1 for even in z direction
		       fp); 
      }
    }

    //top:
    kFine = pointNumberMeshP[m][2]-1;
    // j even, k even
    for (iFine=1; iFine<=pointNumberMeshP[m][0]-1; iFine=iFine+2){
      for (jFine=1; jFine<=pointNumberMeshP[m][1]-1; jFine=jFine+2){
	
	// left bottom corner index
	i = StartIndex[m][0]-1 + (iFine-1) / 2;
	j = StartIndex[m][1]-1 + (jFine-1) / 2;
	k = StartIndex[m][2] + cellNumberPerPatch;

	fineGhostPoint(1, // 0 for odd; 1 for even in x direction
		       1, // 0 for odd; 1 for even in y direction
		       1, // 0 for odd; 1 for even in z direction
		       fp); 
      }     
    }

  
  /*        
  for (m=0; m<=nPatch; m++) {
    for (j=0; j<pointNumberMeshP[m][1]; j++) {
      for (i=0; i<pointNumberMeshP[m][0]; i++) {
	cout << fp[m][i][j] <<" ";
      }
      cout <<endl;
    }
    cout <<"========================="<<endl;
  }
  */      
  
  } // for m

}


void divUStar::coarseGhostPoint(real ****fp){

  fp[0][i][j][k] = ( fp[m][iFine][jFine][kFine]         //WSB
		     + fp[m][iFine+1][jFine][kFine]     //ESB
		     + fp[m][iFine][jFine+1][kFine]     //WNB
		     + fp[m][iFine+1][jFine+1][kFine]   //ENB
		     + fp[m][iFine][jFine][kFine+1]     //WST
		     + fp[m][iFine+1][jFine][kFine+1]   //EST
		     + fp[m][iFine][jFine+1][kFine+1]   //WNT
		     + fp[m][iFine+1][jFine+1][kFine+1] //ENT
		     ) / 8.0;

}

void divUStar::fineGhostPoint(real ix, // 0 for odd; 1 for even in x direction
			  real iy, // 0 for odd; 1 for even in y direction
			  real iz, // 0 for odd; 1 for even in z direction
			  real ****fp){

  fp[m][iFine][jFine][kFine] = ( pow(3.0, fabs(ix) + fabs(iy) + fabs(iz))
				 * fp[0][i][j][k]       //WSB
				 + pow(3.0, fabs(ix - 1) + fabs(iy) + fabs(iz))
				 * fp[0][i+1][j][k]     //ESB
				 + pow(3.0, fabs(ix) + fabs(iy - 1) + fabs(iz))
				 * fp[0][i][j+1][k]     //WNB
				 + pow(3.0, fabs(ix - 1) + fabs(iy - 1) + fabs(iz))
				 * fp[0][i+1][j+1][k]   //ENB
				 + pow(3.0, fabs(ix) + fabs(iy) + fabs(iz - 1))
				 * fp[0][i][j][k+1]     //WST
				 + pow(3.0, fabs(ix - 1) + fabs(iy) + fabs(iz - 1))
				 * fp[0][i+1][j][k+1]   //EST
				 + pow(3.0, fabs(ix) + fabs(iy - 1) + fabs(iz - 1))
				 * fp[0][i][j+1][k+1]   //WNT
				 + pow(3.0, fabs(ix - 1) + fabs(iy - 1) + fabs(iz - 1))
				 * fp[0][i+1][j+1][k+1] //ENT
				 ) / 64.0;
}


divUStar::~divUStar(){
}
