#ifndef DIVUSTAR_H
#define DIVUSTAR_H

/** \file divUStar.cpp
    \brief Calculate divergence of u*

   Calculate divergence of the predicted velocity at all pressure points on the coarse and fine meshes. They also include the interpolation of the points at the interface between two meshes. 

*/

#include <iostream>
#include <string>
#include <cmath>
using namespace std;

#ifdef SINGLE
#define real float
#else
#define real double
#endif

/** 
    \brief Calculate divergence of u*

   Calculate divergence of the predicted velocity at all pressure points on the coarse and fine meshes. They also include the interpolation of the points at the interface between two meshes. 

*/

class divUStar {

 public:
  /**
     constructor
   */
  divUStar();

/** 
    \brief Calculate divergence of u*

   Calculate divergence of the predicted velocity at all pressure points on the coarse and fine meshes. They also include the interpolation of the points at the interface between two meshes. 

*/
  void initialize(int numberOfPatches, /**< number of patches */
		  real dt, /**< time increment */
		  int cellNumberPerPatch, /**<  cell numbers per patch surrounding the sphere*/
		  int ** StartIndex,  /**< used to put the refined mesh */
		  real ****fp, /**< force for pressure poisson equation */
		  int ** pointNumberMeshP, /**< mesh points for pressure */
		  real *h, /**< space step */
		  real ***xP, /**< coordinates of pressure */
		  real ****uAux, /**< predicted u */
		  real ****vAux, /**< predicted v */
		  real ****wAux /**< predicted w */
		  );

  /**
     destructor
   */
  ~divUStar();
  
 private: 

  void coarseGhostPoint(real ****fp);
  void fineGhostPoint(real ix, // 0 for odd; 1 for even in x direction
		      real iy, // 0 for odd; 1 for even in y direction
		      real iz, // 0 for odd; 1 for even in z direction
		      real ****fp);

  int nPatch; // each patch carries a square fine mesh
  int nMeshes;   // including the coarse mesh
  // the coarse mesh is the zeroth mesh and fine meshes are from 1st to the last
  int nCellPerPatch; // cell number in the coarse mesh that a patch uses

  int dim;

  int i, j, k;

  int iFine, jFine, kFine;

  // m for mesh iteration: meshes for patchs and coarse grid
  int m;



};

#endif
