#ifndef EXACTSOL_H
#define EXACTSOL_H

/** \file exactSol.cpp
    \brief set exact solutions of velocity and pressure for calculating accuracy

*/

#include <cmath>
using namespace std;

#ifdef SINGLE
#define real float
#else
#define real double
#endif

/** 
    \brief set exact solutions of velocity and pressure for calculating accuracy

*/
class exactSol {

 public:
  /**
     constructor
   */
  exactSol();

/** \file exactSol.cpp
    \brief set exact solutions of velocity and pressure for calculating accuracy

*/
  void initialize(int nMeshes, /**< number of meshs */
		  real t, /**<  time */
		  int ** pointNumberMeshU, /**< mesh points for u */
		  real ***xU, /**< coordinates of u  */
		  int ** pointNumberMeshV, /**< mesh points for v */
		  real ***xV, /**< coordinates of v  */
		  int ** pointNumberMeshW, /**< mesh points for w */
		  real ***xW, /**< coordinates of w  */
		  int ** pointNumberMeshP, /**< mesh points for p */
		  real ***xP,  /**< coordinates of pressure */
		  real ****exactU, /**< exact solutions of u */
		  real ****exactV, /**< exact solutions of v */
		  real ****exactW, /**< exact solutions of w */
		  real ****exactP  /**< exact solutions of p */
		  );

  /**
     destructor
   */
  ~exactSol();
  
 private: 


  // iRHS for index in matrix A = totalUnknowns x totalUnknowns
  // jRHS
  int i, j, k;


  // m for mesh iteration: meshes for patchs and coarse grid
  int m;

  // nx and ny for wavenumber in x and y directions
  int nx, ny, nz;


};

#endif
