#include "ghostPoint.h"


ghostPoint::ghostPoint(){
}

void ghostPoint::initialize(int numberOfPatches,
			    real t,
			    real dt,
			    int cellNumberPerPatch, 
			    int ** StartIndex, 
			    int ** pointNumberMeshU, 
			    int ** pointNumberMeshV, 
			    int ** pointNumberMeshW, 
			    int dimension,
			    real ***xU, 
			    real ***xV, 
			    real ***xW, 
			    real *h,
			    real ****u,
			    real ****v,
			    real ****w)
{

  nPatch = numberOfPatches;
  nMeshes = numberOfPatches + 1;
  nCellPerPatch = cellNumberPerPatch;
  dim = dimension;


  for (m=1; m<=nPatch; m++) {

    //////////////////////////////////////////////
    //
    //interface points for the fine mesh
    // values from coarse mesh
    //
    //////////////////////////////////////////////

    
    ///////////////////////////////////////////////
    //
    //        for u
    //    
    //////////////////////////////////////////////

    
    ///////////  x direction   

    //west
    //j odd, k odd
    i = 0;
    for (k=0; k<=pointNumberMeshU[m][2]-2; k=k+2) {
      for (j=0; j<=pointNumberMeshU[m][1]-2; j=j+2) {

	iCoarse = StartIndex[m][0]-1; // (iCoarse, jCoarse), index of bottom coarse point
	jCoarse = StartIndex[m][1]-1 + j/2;
	kCoarse = StartIndex[m][2]-1 + k/2;

	uFineGhostPoint(0, // 0 for odd; 1 for even in y direction
			0, // 0 for odd; 1 for even in z direction
			u);	
      }
    }

    //j odd, k even
    i = 0;
    for (k=1; k<=pointNumberMeshU[m][2]-1; k=k+2) {
      for (j=0; j<=pointNumberMeshU[m][1]-2; j=j+2) {

	iCoarse = StartIndex[m][0]-1; // (iCoarse, jCoarse), index of bottom coarse point
	jCoarse = StartIndex[m][1]-1 + j/2;
	kCoarse = StartIndex[m][2] + (k-1)/2;

	uFineGhostPoint(0, // 0 for odd; 1 for even in y direction
			1, // 0 for odd; 1 for even in z direction
			u);	
      }
    }

    //j even, k odd
    i = 0;
    for (k=0; k<=pointNumberMeshU[m][2]-2; k=k+2) {
      for (j=1; j<=pointNumberMeshU[m][1]-1; j=j+2) {

	iCoarse = StartIndex[m][0]-1; // (iCoarse, jCoarse), index of bottom coarse point
	jCoarse = StartIndex[m][1] + (j-1)/2;
	kCoarse = StartIndex[m][2]-1 + k/2;

	uFineGhostPoint(1, // 0 for odd; 1 for even in y direction
			0, // 0 for odd; 1 for even in z direction
			u);	
      }
    }

  // j even, k even
    i = 0;
    for (k=1; k<=pointNumberMeshU[m][2]-1; k=k+2) {
      for (j=1; j<=pointNumberMeshU[m][1]-1; j=j+2) {
	iCoarse = StartIndex[m][0]-1; // (iCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1] + (j-1)/2;
	kCoarse = StartIndex[m][2] + (k-1)/2;
	
	uFineGhostPoint(1, // 0 for odd; 1 for even in y direction
			1, // 0 for odd; 1 for even in z direction
			u);	
      }
    }



    //east
    //j odd, k odd
    i = pointNumberMeshU[m][0]-1;
    for (k=0; k<=pointNumberMeshU[m][2]-2; k=k+2) {
      for (j=0; j<=pointNumberMeshU[m][1]-2; j=j+2) {
	
	iCoarse = StartIndex[m][0]+1 + cellNumberPerPatch;  // (iCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1]-1 + j/2;
	kCoarse = StartIndex[m][2]-1 + k/2;

	uFineGhostPoint(0, // 0 for odd; 1 for even in y direction
			0, // 0 for odd; 1 for even in z direction
			u);	
      }
    }


    //j odd, k even
    i = pointNumberMeshU[m][0]-1;
    for (k=1; k<=pointNumberMeshU[m][2]-1; k=k+2) {
      for (j=0; j<=pointNumberMeshU[m][1]-2; j=j+2) {

	iCoarse = StartIndex[m][0]+1 + cellNumberPerPatch;  // (iCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1]-1 + j/2;
	kCoarse = StartIndex[m][2] + (k-1)/2;

	uFineGhostPoint(0, // 0 for odd; 1 for even in y direction
			1, // 0 for odd; 1 for even in z direction
			u);	
      }
    }

    //j even, k odd
    i = pointNumberMeshU[m][0]-1;
    for (k=0; k<=pointNumberMeshU[m][2]-2; k=k+2) {
      for (j=1; j<=pointNumberMeshU[m][1]-1; j=j+2) {

	iCoarse = StartIndex[m][0]+1 + cellNumberPerPatch;  // (iCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1] + (j-1)/2;
	kCoarse = StartIndex[m][2]-1 + k/2;

	uFineGhostPoint(1, // 0 for odd; 1 for even in y direction
			0, // 0 for odd; 1 for even in z direction
			u);	
      }
    }

  // j even, k even
    i = pointNumberMeshU[m][0]-1;
    for (k=1; k<=pointNumberMeshU[m][2]-1; k=k+2) {
      for (j=1; j<=pointNumberMeshU[m][1]-1; j=j+2) {

	iCoarse = StartIndex[m][0]+1 + cellNumberPerPatch;  // (iCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1] + (j-1)/2;
	kCoarse = StartIndex[m][2] + (k-1)/2;
	
	uFineGhostPoint(1, // 0 for odd; 1 for even in y direction
			1, // 0 for odd; 1 for even in z direction
			u);	
      }
    }


    ///////////  y direction   


  //south: i odd
  // j odd, k odd 
    j = 0;
    for (i=0; i<=pointNumberMeshU[m][0]-1; i=i+2) {
      for (k=0; k<=pointNumberMeshU[m][2]-1; k=k+2) {
	iCoarse = StartIndex[m][0]-1 + i/2; // (iCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1]-1;
	kCoarse = StartIndex[m][2]-1 + k/2; 
	
	uFineGhostPoint(0, // 0 for odd; 1 for even in y direction
			0, // 0 for odd; 1 for even in z direction
			u);	
      }
    }

  // j odd, k even 
    j = 0;
    for (k=1; k<=pointNumberMeshU[m][2]-1; k=k+2) {
      for (i=0; i<=pointNumberMeshU[m][0]-1; i=i+2) {
	iCoarse = StartIndex[m][0]-1 + i/2; // (iCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1]-1;
	kCoarse = StartIndex[m][2] + (k-1)/2;
	
	uFineGhostPoint(0, // 0 for odd; 1 for even in y direction
			1, // 0 for odd; 1 for even in z direction
			u);	
      }
    }


  //i even
    j = 0;
    for (k=0; k<=pointNumberMeshU[m][2]-1; k=k+1) {
      for (i=1; i<=pointNumberMeshU[m][0]-2; i=i+2) {
	
	//approach one
	u[m][i][j][k] = ( u[m][i-1][j][k] + u[m][i+1][j][k] ) / 2.0; 
	
      }
    }



  //north: i odd
  // j even, k odd 
    j = pointNumberMeshU[m][1]-1;
    for (i=0; i<=pointNumberMeshU[m][0]-1; i=i+2) {
      for (k=0; k<=pointNumberMeshU[m][2]-1; k=k+2) {
	iCoarse = StartIndex[m][0]-1 + i/2; // (iCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1] + cellNumberPerPatch + 1;
	kCoarse = StartIndex[m][2]-1 + k/2; 
	
	uFineGhostPoint(1, // 0 for odd; 1 for even in y direction
			0, // 0 for odd; 1 for even in z direction
			u);	
      }
    }

  // j even, k even 
    j = pointNumberMeshU[m][1]-1;
    for (k=1; k<=pointNumberMeshU[m][2]-1; k=k+2) {
      for (i=0; i<=pointNumberMeshU[m][0]-1; i=i+2) {
	iCoarse = StartIndex[m][0]-1 + i/2; // (iCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1] + cellNumberPerPatch + 1;
	kCoarse = StartIndex[m][2] + (k-1)/2;
	
	uFineGhostPoint(1, // 0 for odd; 1 for even in y direction
			1, // 0 for odd; 1 for even in z direction
			u);	
      }
    }


  //i even
    j = pointNumberMeshU[m][1] - 1;
    for (k=0; k<=pointNumberMeshU[m][2]-1; k=k+1) {
      for (i=1; i<=pointNumberMeshU[m][0]-2; i=i+2) {
	
	//approach one
	u[m][i][j][k] = ( u[m][i-1][j][k] + u[m][i+1][j][k] ) / 2.0; 
	
      }
    }


    ///////////  z direction   


  //south: i odd
  // k odd, j odd 
    k = 0;
    for (i=0; i<=pointNumberMeshU[m][0]-1; i=i+2) {
      for (j=0; j<=pointNumberMeshU[m][1]-1; j=j+2) {
	iCoarse = StartIndex[m][0]-1 + i/2; // (iCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1]-1 + j/2; 
	kCoarse = StartIndex[m][2]-1;
	
	uFineGhostPoint(0, // 0 for odd; 1 for even in y direction
			0, // 0 for odd; 1 for even in z direction
			u);	
      }
    }

  // k odd, j even 
    k = 0;
    for (j=1; j<=pointNumberMeshU[m][1]-1; j=j+2) {
      for (i=0; i<=pointNumberMeshU[m][0]-1; i=i+2) {
	iCoarse = StartIndex[m][0]-1 + i/2; // (iCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1] + (j-1)/2;
	kCoarse = StartIndex[m][2]-1;
	
	uFineGhostPoint(1, // 0 for odd; 1 for even in y direction
			0, // 0 for odd; 1 for even in z direction
			u);	
      }
    }


  //i even
    k = 0;
    for (j=0; j<=pointNumberMeshU[m][1]-1; j=j+1) {
      for (i=1; i<=pointNumberMeshU[m][0]-2; i=i+2) {
	
	//approach one
	u[m][i][j][k] = ( u[m][i-1][j][k] + u[m][i+1][j][k] ) / 2.0; 
	
      }
    }




  //north: i odd
  // k even, j odd 
    k = pointNumberMeshU[m][2]-1;
    for (i=0; i<=pointNumberMeshU[m][0]-1; i=i+2) {
      for (j=0; j<=pointNumberMeshU[m][1]-1; j=j+2) {
	iCoarse = StartIndex[m][0]-1 + i/2; // (iCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1]-1 + j/2; 
	kCoarse = StartIndex[m][2] + cellNumberPerPatch + 1;
	
	uFineGhostPoint(0, // 0 for odd; 1 for even in y direction
			1, // 0 for odd; 1 for even in z direction
			u);	
      }
    }

  // k even, j even 
    k = pointNumberMeshU[m][2]-1;
    for (j=1; j<=pointNumberMeshU[m][1]-1; j=j+2) {
      for (i=0; i<=pointNumberMeshU[m][0]-1; i=i+2) {
	iCoarse = StartIndex[m][0]-1 + i/2; // (iCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1] + (j-1)/2;
	kCoarse = StartIndex[m][2] + cellNumberPerPatch + 1;
	
	uFineGhostPoint(1, // 0 for odd; 1 for even in y direction
			1, // 0 for odd; 1 for even in z direction
			u);	
      }
    }


  //i even
    k = pointNumberMeshU[m][2] - 1;
    for (j=0; j<=pointNumberMeshU[m][1]-1; j=j+1) {
      for (i=1; i<=pointNumberMeshU[m][0]-2; i=i+2) {
	
	//approach one
	u[m][i][j][k] = ( u[m][i-1][j][k] + u[m][i+1][j][k] ) / 2.0; 
	
      }
    }

 


    ///////////////////////////////////////////////
    //
    //        for v
    //    
    //
    //////////////////////////////////////////////
    
    // from u
    // switch u and v
    // switch the formula of indices for i and j
    // switch x and y
    // switch west and south
    // switch east and north

    
    ///////////  y direction   

    //south
    //i odd, k odd
    j = 0;
    for (k=0; k<=pointNumberMeshV[m][2]-2; k=k+2) {
      for (i=0; i<=pointNumberMeshV[m][0]-2; i=i+2) {

	jCoarse = StartIndex[m][1]-1; // (iCoarse, jCoarse), index of bottom coarse point
	iCoarse = StartIndex[m][0]-1 + i/2;
	kCoarse = StartIndex[m][2]-1 + k/2;

	vFineGhostPoint(0, // 0 for odd; 1 for even in x direction
			0, // 0 for odd; 1 for even in z direction
			v);	
      }
    }

    //i odd, k even
    j = 0;
    for (k=1; k<=pointNumberMeshV[m][2]-1; k=k+2) {
      for (i=0; i<=pointNumberMeshV[m][0]-2; i=i+2) {

	jCoarse = StartIndex[m][1]-1; // (iCoarse, jCoarse), index of bottom coarse point
	iCoarse = StartIndex[m][0]-1 + i/2;
	kCoarse = StartIndex[m][2] + (k-1)/2;

	vFineGhostPoint(0, // 0 for odd; 1 for even in x direction
			1, // 0 for odd; 1 for even in z direction
			v);	
      }
    }

    //i even, k odd
    j = 0;
    for (k=0; k<=pointNumberMeshV[m][2]-2; k=k+2) {
      for (i=1; i<=pointNumberMeshV[m][0]-1; i=i+2) {

	jCoarse = StartIndex[m][1]-1; // (iCoarse, jCoarse), index of bottom coarse point
	iCoarse = StartIndex[m][0] + (i-1)/2;
	kCoarse = StartIndex[m][2]-1 + k/2;

	vFineGhostPoint(1, // 0 for odd; 1 for even in x direction
			0, // 0 for odd; 1 for even in z direction
			v);	
      }
    }

  // i even, k even
    j = 0;
    for (k=1; k<=pointNumberMeshV[m][2]-1; k=k+2) {
      for (i=1; i<=pointNumberMeshV[m][0]-1; i=i+2) {
	jCoarse = StartIndex[m][1]-1; // (iCoarse, jCoarse), index of left bottom coarse point
	iCoarse = StartIndex[m][0] + (i-1)/2;
	kCoarse = StartIndex[m][2] + (k-1)/2;
	
	vFineGhostPoint(1, // 0 for odd; 1 for even in x direction
			1, // 0 for odd; 1 for even in z direction
			v);	
      }
    }



    //north
    //i odd, k odd
    j = pointNumberMeshV[m][1]-1;
    for (k=0; k<=pointNumberMeshV[m][2]-2; k=k+2) {
      for (i=0; i<=pointNumberMeshV[m][0]-2; i=i+2) {
	
	jCoarse = StartIndex[m][1]+1 + cellNumberPerPatch;  // (iCoarse, jCoarse), index of left bottom coarse point
	iCoarse = StartIndex[m][0]-1 + i/2;
	kCoarse = StartIndex[m][2]-1 + k/2;

	vFineGhostPoint(0, // 0 for odd; 1 for even in x direction
			0, // 0 for odd; 1 for even in z direction
			v);	
      }
    }


    //i odd, k even
    j = pointNumberMeshV[m][1]-1;
    for (k=1; k<=pointNumberMeshV[m][2]-1; k=k+2) {
      for (i=0; i<=pointNumberMeshV[m][0]-2; i=i+2) {

	jCoarse = StartIndex[m][1]+1 + cellNumberPerPatch;  // (iCoarse, jCoarse), index of left bottom coarse point
	iCoarse = StartIndex[m][0]-1 + i/2;
	kCoarse = StartIndex[m][2] + (k-1)/2;

	vFineGhostPoint(0, // 0 for odd; 1 for even in x direction
			1, // 0 for odd; 1 for even in z direction
			v);	
      }
    }

    //i even, k odd
    j = pointNumberMeshV[m][1]-1;
    for (k=0; k<=pointNumberMeshV[m][2]-2; k=k+2) {
      for (i=1; i<=pointNumberMeshV[m][0]-1; i=i+2) {

	jCoarse = StartIndex[m][1]+1 + cellNumberPerPatch;  // (iCoarse, jCoarse), index of left bottom coarse point
	iCoarse = StartIndex[m][0] + (i-1)/2;
	kCoarse = StartIndex[m][2]-1 + k/2;

	vFineGhostPoint(1, // 0 for odd; 1 for even in x direction
			0, // 0 for odd; 1 for even in z direction
			v);	
      }
    }

  // i even, k even
    j = pointNumberMeshV[m][1]-1;
    for (k=1; k<=pointNumberMeshV[m][2]-1; k=k+2) {
      for (i=1; i<=pointNumberMeshV[m][0]-1; i=i+2) {

	jCoarse = StartIndex[m][1]+1 + cellNumberPerPatch;  // (iCoarse, jCoarse), index of left bottom coarse point
	iCoarse = StartIndex[m][0] + (i-1)/2;
	kCoarse = StartIndex[m][2] + (k-1)/2;
	
	vFineGhostPoint(1, // 0 for odd; 1 for even in x direction
			1, // 0 for odd; 1 for even in z direction
			v);	
      }
    }


    ///////////  x direction   
    

    //west: j odd
    // i odd, k odd 
    i = 0;
    for (j=0; j<=pointNumberMeshV[m][1]-1; j=j+2) {
      for (k=0; k<=pointNumberMeshV[m][2]-1; k=k+2) {
	jCoarse = StartIndex[m][1]-1 + j/2; // (iCoarse, jCoarse), index of left bottom coarse point
	iCoarse = StartIndex[m][0]-1;
	kCoarse = StartIndex[m][2]-1 + k/2; 
	
	vFineGhostPoint(0, // 0 for odd; 1 for even in x direction
			0, // 0 for odd; 1 for even in z direction
			v);	
      }
    }

  // i odd, k even 
    i = 0;
    for (k=1; k<=pointNumberMeshV[m][2]-1; k=k+2) {
      for (j=0; j<=pointNumberMeshV[m][1]-1; j=j+2) {
	jCoarse = StartIndex[m][1]-1 + j/2; // (iCoarse, jCoarse), index of left bottom coarse point
	iCoarse = StartIndex[m][0]-1;
	kCoarse = StartIndex[m][2] + (k-1)/2;
	
	vFineGhostPoint(0, // 0 for odd; 1 for even in x direction
			1, // 0 for odd; 1 for even in z direction
			v);	
      }
    }


  //j even
    i = 0;
    for (k=0; k<=pointNumberMeshV[m][2]-1; k=k+1) {
      for (j=1; j<=pointNumberMeshV[m][1]-2; j=j+2) {
	
	//approach one
	v[m][i][j][k] = ( v[m][i][j-1][k] + v[m][i][j+1][k] ) / 2.0; 
	
      }
    }




  //east: j odd
  // i even, k odd 
    i = pointNumberMeshV[m][0]-1;
    for (j=0; j<=pointNumberMeshV[m][1]-1; j=j+2) {
      for (k=0; k<=pointNumberMeshV[m][2]-1; k=k+2) {
	jCoarse = StartIndex[m][1]-1 + j/2; // (iCoarse, jCoarse), index of left bottom coarse point
	iCoarse = StartIndex[m][0] + cellNumberPerPatch + 1;
	kCoarse = StartIndex[m][2]-1 + k/2; 
	
	vFineGhostPoint(1, // 0 for odd; 1 for even in x direction
			0, // 0 for odd; 1 for even in z direction
			v);	
      }
    }

  // i even, k even 
    i = pointNumberMeshV[m][0]-1;
    for (k=1; k<=pointNumberMeshV[m][2]-1; k=k+2) {
      for (j=0; j<=pointNumberMeshV[m][1]-1; j=j+2) {
	jCoarse = StartIndex[m][1]-1 + j/2; // (iCoarse, jCoarse), index of left bottom coarse point
	iCoarse = StartIndex[m][0] + cellNumberPerPatch + 1;
	kCoarse = StartIndex[m][2] + (k-1)/2;
	
	vFineGhostPoint(1, // 0 for odd; 1 for even in x direction
			1, // 0 for odd; 1 for even in z direction
			v);	
      }
    }


  //j even
    i = pointNumberMeshV[m][0] - 1;
    for (k=0; k<=pointNumberMeshV[m][2]-1; k=k+1) {
      for (j=1; j<=pointNumberMeshV[m][1]-2; j=j+2) {
	
	//approach one
	v[m][i][j][k] = ( v[m][i][j-1][k] + v[m][i][j+1][k] ) / 2.0; 
	
      }
    }


    ///////////  z direction   


  //south: j odd
  // k odd, i odd 
    k = 0;
    for (i=0; i<=pointNumberMeshV[m][0]-1; i=i+2) {
      for (j=0; j<=pointNumberMeshV[m][1]-1; j=j+2) {
	iCoarse = StartIndex[m][0]-1 + i/2; // (iCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1]-1 + j/2; 
	kCoarse = StartIndex[m][2]-1;
	
	vFineGhostPoint(0, // 0 for odd; 1 for even in x direction
			0, // 0 for odd; 1 for even in z direction
			v);	
      }
    }

  // k odd, i even 
    k = 0;
    for (i=1; i<=pointNumberMeshV[m][0]-1; i=i+2) {
      for (j=0; j<=pointNumberMeshV[m][1]-1; j=j+2) {
	jCoarse = StartIndex[m][1]-1 + j/2; // (iCoarse, jCoarse), index of left bottom coarse point
	iCoarse = StartIndex[m][0] + (i-1)/2;
	kCoarse = StartIndex[m][2]-1;
	
	vFineGhostPoint(1, // 0 for odd; 1 for even in x direction
			0, // 0 for odd; 1 for even in z direction
			v);	
      }
    }


  //j even
    k = 0;
    for (i=0; i<=pointNumberMeshV[m][0]-1; i=i+1) {
      for (j=1; j<=pointNumberMeshV[m][1]-2; j=j+2) {
	
	//approach one
	v[m][i][j][k] = ( v[m][i][j-1][k] + v[m][i][j+1][k] ) / 2.0; 
	
      }
    }




  //north: j odd
  // k even, i odd 
    k = pointNumberMeshV[m][2]-1;
    for (i=0; i<=pointNumberMeshV[m][0]-1; i=i+2) {
      for (j=0; j<=pointNumberMeshV[m][1]-1; j=j+2) {
	iCoarse = StartIndex[m][0]-1 + i/2; // (iCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1]-1 + j/2; 
	kCoarse = StartIndex[m][2] + cellNumberPerPatch + 1;
	
	vFineGhostPoint(0, // 0 for odd; 1 for even in x direction
			1, // 0 for odd; 1 for even in z direction
			v);	
      }
    }

  // k even, i even 
    k = pointNumberMeshV[m][2]-1;
    for (i=1; i<=pointNumberMeshV[m][0]-1; i=i+2) {
      for (j=0; j<=pointNumberMeshV[m][1]-1; j=j+2) {
	jCoarse = StartIndex[m][1]-1 + j/2; // (iCoarse, jCoarse), index of left bottom coarse point
	iCoarse = StartIndex[m][0] + (i-1)/2;
	kCoarse = StartIndex[m][2] + cellNumberPerPatch + 1;
	
	vFineGhostPoint(1, // 0 for odd; 1 for even in x direction
			1, // 0 for odd; 1 for even in z direction
			v);	
      }
    }


  //j even
    k = pointNumberMeshV[m][2] - 1;
    for (i=0; i<=pointNumberMeshV[m][0]-1; i=i+1) {
      for (j=1; j<=pointNumberMeshV[m][1]-2; j=j+2) {
	
	//approach one
	v[m][i][j][k] = ( v[m][i][j-1][k] + v[m][i][j+1][k] ) / 2.0; 
	
      }
    }





    ///////////////////////////////////////////////
    //
    //        for w
    //    
    //
    //////////////////////////////////////////////

    
    ///////////  x direction   

    //bottom
    //j odd, i odd
    k = 0;
    for (i=0; i<=pointNumberMeshW[m][0]-2; i=i+2) {
      for (j=0; j<=pointNumberMeshW[m][1]-2; j=j+2) {

	kCoarse = StartIndex[m][2]-1; // (kCoarse, jCoarse), index of bottom coarse point
	jCoarse = StartIndex[m][1]-1 + j/2;
	iCoarse = StartIndex[m][0]-1 + i/2;

	wFineGhostPoint(0, // 0 for odd; 1 for even in x direction
			0, // 0 for odd; 1 for even in y direction
			w);	
      }
    }

    //j odd, k even
    k = 0;
    for (i=1; i<=pointNumberMeshW[m][0]-1; i=i+2) {
      for (j=0; j<=pointNumberMeshW[m][1]-2; j=j+2) {

	kCoarse = StartIndex[m][2]-1; // (kCoarse, jCoarse), index of bottom coarse point
	jCoarse = StartIndex[m][1]-1 + j/2;
	iCoarse = StartIndex[m][0] + (i-1)/2;

	wFineGhostPoint(1, // 0 for odd; 1 for even in x direction
			0, // 0 for odd; 1 for even in y direction
			w);	
      }
    }

    //j even, k odd
    k = 0;
    for (i=0; i<=pointNumberMeshW[m][0]-2; i=i+2) {
      for (j=1; j<=pointNumberMeshW[m][1]-1; j=j+2) {

	kCoarse = StartIndex[m][2]-1; // (kCoarse, jCoarse), index of bottom coarse point
	jCoarse = StartIndex[m][1] + (j-1)/2;
	iCoarse = StartIndex[m][0]-1 + i/2;

	wFineGhostPoint(0, // 0 for odd; 1 for even in x direction
			1, // 0 for odd; 1 for even in y direction
			w);	
      }
    }

  // j even, k even
    k = 0;
    for (i=1; i<=pointNumberMeshW[m][0]-1; i=i+2) {
      for (j=1; j<=pointNumberMeshW[m][1]-1; j=j+2) {
	kCoarse = StartIndex[m][2]-1; // (kCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1] + (j-1)/2;
	iCoarse = StartIndex[m][0] + (i-1)/2;
	
	wFineGhostPoint(1, // 0 for odd; 1 for even in x direction
			1, // 0 for odd; 1 for even in y direction
			w);	
      }
    }



    //east
    //j odd, i odd
    k = pointNumberMeshW[m][2]-1;
    for (i=0; i<=pointNumberMeshW[m][0]-2; i=i+2) {
      for (j=0; j<=pointNumberMeshW[m][1]-2; j=j+2) {
	
	kCoarse = StartIndex[m][2]+1 + cellNumberPerPatch;  // (kCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1]-1 + j/2;
	iCoarse = StartIndex[m][0]-1 + i/2;

	wFineGhostPoint(0, // 0 for odd; 1 for even in x direction
			0, // 0 for odd; 1 for even in y direction
			w);	
      }
    }


    //j odd, i even
    k = pointNumberMeshW[m][2]-1;
    for (i=1; i<=pointNumberMeshW[m][0]-1; i=i+2) {
      for (j=0; j<=pointNumberMeshW[m][1]-2; j=j+2) {

	kCoarse = StartIndex[m][2]+1 + cellNumberPerPatch;  // (kCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1]-1 + j/2;
	iCoarse = StartIndex[m][0] + (i-1)/2;

	wFineGhostPoint(1, // 0 for odd; 1 for even in x direction
			0, // 0 for odd; 1 for even in y direction
			w);	
      }
    }

    //j even, i odd
    k = pointNumberMeshW[m][2]-1;
    for (i=0; i<=pointNumberMeshW[m][0]-2; i=i+2) {
      for (j=1; j<=pointNumberMeshW[m][1]-1; j=j+2) {

	kCoarse = StartIndex[m][2]+1 + cellNumberPerPatch;  // (kCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1] + (j-1)/2;
	iCoarse = StartIndex[m][0]-1 + i/2;

	wFineGhostPoint(0, // 0 for odd; 1 for even in x direction
			1, // 0 for odd; 1 for even in y direction
			w);	
      }
    }

  // j even, i even
    k = pointNumberMeshW[m][2]-1;
    for (i=1; i<=pointNumberMeshW[m][0]-1; i=i+2) {
      for (j=1; j<=pointNumberMeshW[m][1]-1; j=j+2) {

	kCoarse = StartIndex[m][2]+1 + cellNumberPerPatch;  // (kCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1] + (j-1)/2;
	iCoarse = StartIndex[m][0] + (i-1)/2;
	
	wFineGhostPoint(1, // 0 for odd; 1 for even in x direction
			1, // 0 for odd; 1 for even in y direction
			w);	
      }
    }


    ///////////  y direction   


  //south: k odd
  // j odd, i odd 
    j = 0;
    for (i=0; i<=pointNumberMeshW[m][0]-1; i=i+2) {
      for (k=0; k<=pointNumberMeshW[m][2]-1; k=k+2) {
	kCoarse = StartIndex[m][2]-1 + k/2; // (kCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1]-1;
	iCoarse = StartIndex[m][0]-1 + i/2; 
	
	wFineGhostPoint(0, // 0 for odd; 1 for even in x direction
			0, // 0 for odd; 1 for even in y direction
			w);	
      }
    }

  // j odd, i even 
    j = 0;
    for (i=1; i<=pointNumberMeshW[m][0]-1; i=i+2) {
      for (k=0; k<=pointNumberMeshW[m][2]-1; k=k+2) {
	kCoarse = StartIndex[m][2]-1 + k/2; // (kCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1]-1;
	iCoarse = StartIndex[m][0] + (i-1)/2;
	
	wFineGhostPoint(1, // 0 for odd; 1 for even in x direction
			0, // 0 for odd; 1 for even in y direction
			w);	
      }
    }


  //i even
    j = 0;
    for (i=0; i<=pointNumberMeshW[m][0]-1; i=i+1) {
      for (k=1; k<=pointNumberMeshW[m][2]-2; k=k+2) {
	
	//approach one
	w[m][i][j][k] = ( w[m][i][j][k-1] + w[m][i][j][k+1] ) / 2.0; 
	
      }
    }




  //north: k odd
  // j even, i odd 
    j = pointNumberMeshW[m][1]-1;
    for (i=0; i<=pointNumberMeshW[m][0]-1; i=i+2) {
      for (k=0; k<=pointNumberMeshW[m][2]-1; k=k+2) {
	kCoarse = StartIndex[m][2]-1 + k/2; // (kCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1] + cellNumberPerPatch + 1;
	iCoarse = StartIndex[m][0]-1 + i/2; 

	wFineGhostPoint(0, // 0 for odd; 1 for even in x direction
			1, // 0 for odd; 1 for even in y direction
			w);	
      }
    }

  // j even, i even 
    j = pointNumberMeshW[m][1]-1;
    for (i=1; i<=pointNumberMeshW[m][0]-1; i=i+2) {
      for (k=0; k<=pointNumberMeshW[m][2]-1; k=k+2) {
	kCoarse = StartIndex[m][2]-1 + k/2; // (kCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1] + cellNumberPerPatch + 1;
	iCoarse = StartIndex[m][0] + (i-1)/2;
	
	wFineGhostPoint(1, // 0 for odd; 1 for even in x direction
			1, // 0 for odd; 1 for even in y direction
			w);	
      }
    }


  //k even
    j = pointNumberMeshW[m][1] - 1;
    for (i=0; i<=pointNumberMeshW[m][0]-1; i=i+1) {
      for (k=1; k<=pointNumberMeshW[m][2]-2; k=k+2) {
	
	//approach one
	w[m][i][j][k] = ( w[m][i][j][k-1] + w[m][i][j][k+1] ) / 2.0; 
	
      }
    }


    ///////////  z direction   


  //south: k odd
  // i odd, j odd 
    i = 0;
    for (k=0; k<=pointNumberMeshW[m][2]-1; k=k+2) {
      for (j=0; j<=pointNumberMeshW[m][1]-1; j=j+2) {
	kCoarse = StartIndex[m][2]-1 + k/2; // (kCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1]-1 + j/2; 
	iCoarse = StartIndex[m][0]-1;
	
	wFineGhostPoint(0, // 0 for odd; 1 for even in x direction
			0, // 0 for odd; 1 for even in y direction
			w);	
      }
    }

  // i odd, j even 
    i = 0;
    for (j=1; j<=pointNumberMeshW[m][1]-1; j=j+2) {
      for (k=0; k<=pointNumberMeshW[m][2]-1; k=k+2) {
	kCoarse = StartIndex[m][2]-1 + k/2; // (kCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1] + (j-1)/2;
	iCoarse = StartIndex[m][0]-1;
	
	wFineGhostPoint(0, // 0 for odd; 1 for even in x direction
			1, // 0 for odd; 1 for even in y direction
			w);	
      }
    }


  //k even
    i = 0;
    for (j=0; j<=pointNumberMeshW[m][1]-1; j=j+1) {
      for (k=1; k<=pointNumberMeshW[m][2]-2; k=k+2) {
	
	//approach one
	w[m][i][j][k] = ( w[m][i][j][k-1] + w[m][i][j][k+1] ) / 2.0; 
	
      }
    }




  //north: k odd
  // i even, j odd 
    i = pointNumberMeshW[m][0]-1;
    for (k=0; k<=pointNumberMeshW[m][2]-1; k=k+2) {
      for (j=0; j<=pointNumberMeshW[m][1]-1; j=j+2) {
	kCoarse = StartIndex[m][2]-1 + k/2; // (kCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1]-1 + j/2; 
	iCoarse = StartIndex[m][0] + cellNumberPerPatch + 1;
	
	wFineGhostPoint(1, // 0 for odd; 1 for even in x direction
			0, // 0 for odd; 1 for even in y direction
			w);	
      }
    }

  // i even, j even 
    i = pointNumberMeshW[m][0]-1;
    for (j=1; j<=pointNumberMeshW[m][1]-1; j=j+2) {
      for (k=0; k<=pointNumberMeshW[m][2]-1; k=k+2) {
	kCoarse = StartIndex[m][2]-1 + k/2; // (kCoarse, jCoarse), index of left bottom coarse point
	jCoarse = StartIndex[m][1] + (j-1)/2;
	iCoarse = StartIndex[m][0] + cellNumberPerPatch + 1;
	
	wFineGhostPoint(1, // 0 for odd; 1 for even in x direction
			1, // 0 for odd; 1 for even in y direction
			w);	
      }
    }


  //k even
    i = pointNumberMeshW[m][0] - 1;
    for (j=0; j<=pointNumberMeshW[m][1]-1; j=j+1) {
      for (k=1; k<=pointNumberMeshW[m][2]-2; k=k+2) {
	
	//approach one
	w[m][i][j][k] = ( w[m][i][j][k-1] + w[m][i][j][k+1] ) / 2.0; 
	
      }
    }


    //////////////////////////////////////////////
    //
    //interface points for the coarse mesh
    // values from fine mesh
    //
    //////////////////////////////////////////////

    
    ///////////////////////////////////////////////
    //
    //        for u
    //    
    //////////////////////////////////////////////


    //west
    i = StartIndex[m][0] + 1;
    for (k=StartIndex[m][2]+1; k<=StartIndex[m][2]+cellNumberPerPatch; k++) {
      for (j=StartIndex[m][1]+1; j<=StartIndex[m][1]+cellNumberPerPatch; j++) {
	iFine = 0;
	jFine = j - StartIndex[m][1] - 1;
	kFine = k - StartIndex[m][2] - 1;
	
	uCoarseGhostPoint(u);
	
      }
    }

  //east
    i = StartIndex[m][0]+cellNumberPerPatch-1;
    for (k=StartIndex[m][2]+1; k<=StartIndex[m][2]+cellNumberPerPatch; k++) {
      for (j=StartIndex[m][1]+1; j<=StartIndex[m][1]+cellNumberPerPatch; j++) {
	iFine = cellNumberPerPatch - 2;
	jFine = j - StartIndex[m][1] - 1;
	kFine = k - StartIndex[m][2] - 1;
	
	uCoarseGhostPoint(u);
      }
    }


  //south
    j = StartIndex[m][1]+1;
    for (k=StartIndex[m][2]+1; k<=StartIndex[m][2]+cellNumberPerPatch; k++) {
      for (i=StartIndex[m][0]+1; i<=StartIndex[m][0]+cellNumberPerPatch-1; i++) {
	iFine = i - StartIndex[m][0] - 1;
	jFine = 0;
	kFine = k - StartIndex[m][2] - 1;
	
	uCoarseGhostPoint(u);
      }
    }

  //north
    j = StartIndex[m][1]+cellNumberPerPatch;
    for (k=StartIndex[m][2]+1; k<=StartIndex[m][2]+cellNumberPerPatch; k++) {
      for (i=StartIndex[m][0]+1; i<=StartIndex[m][0]+cellNumberPerPatch-1; i++) {
	iFine = i - StartIndex[m][0] - 1;
	jFine = cellNumberPerPatch - 1;
	kFine = k - StartIndex[m][2] - 1;
	
	uCoarseGhostPoint(u);
      }
    }


  //south
    k = StartIndex[m][2]+1;
    for (j=StartIndex[m][1]+1; j<=StartIndex[m][1]+cellNumberPerPatch; j++) {
      for (i=StartIndex[m][0]+1; i<=StartIndex[m][0]+cellNumberPerPatch-1; i++) {
	iFine = i - StartIndex[m][0] - 1;
	kFine = 0;
	jFine = j - StartIndex[m][1] - 1;
	
	uCoarseGhostPoint(u);
      }
    }

  //north
    k = StartIndex[m][2]+cellNumberPerPatch;
    for (j=StartIndex[m][1]+1; j<=StartIndex[m][1]+cellNumberPerPatch; j++) {
      for (i=StartIndex[m][0]+1; i<=StartIndex[m][0]+cellNumberPerPatch-1; i++) {
	iFine = i - StartIndex[m][0] - 1;
	kFine = cellNumberPerPatch - 1;
	jFine = j - StartIndex[m][1] - 1;
	
	uCoarseGhostPoint(u);
	
      }
    }


    ///////////////////////////////////////////////
    //
    //        for v
    //    
    //////////////////////////////////////////////



    //south
    j = StartIndex[m][1] + 1;
    for (k=StartIndex[m][2]+1; k<=StartIndex[m][2]+cellNumberPerPatch; k++) {
      for (i=StartIndex[m][0]+1; i<=StartIndex[m][0]+cellNumberPerPatch; i++) {
	jFine = 0;
	iFine = i - StartIndex[m][0] - 1;
	kFine = k - StartIndex[m][2] - 1;
	
	vCoarseGhostPoint(v);	
      }
    }

  //north
    j = StartIndex[m][1]+cellNumberPerPatch-1;
    for (k=StartIndex[m][2]+1; k<=StartIndex[m][2]+cellNumberPerPatch; k++) {
      for (i=StartIndex[m][0]+1; i<=StartIndex[m][0]+cellNumberPerPatch; i++) {
	jFine = cellNumberPerPatch - 2;
	iFine = i - StartIndex[m][0] - 1;
	kFine = k - StartIndex[m][2] - 1;
	
	vCoarseGhostPoint(v);	
     }
    }


  //west
    i = StartIndex[m][0]+1;
    for (k=StartIndex[m][2]+1; k<=StartIndex[m][2]+cellNumberPerPatch; k++) {
      for (j=StartIndex[m][1]+1; j<=StartIndex[m][1]+cellNumberPerPatch-1; j++) {
	jFine = j - StartIndex[m][1] - 1;
	iFine = 0;
	kFine = k - StartIndex[m][2] - 1;
	
	vCoarseGhostPoint(v);	
     }
    }

  //east
    i = StartIndex[m][0]+cellNumberPerPatch;
    for (k=StartIndex[m][2]+1; k<=StartIndex[m][2]+cellNumberPerPatch; k++) {
      for (j=StartIndex[m][1]+1; j<=StartIndex[m][1]+cellNumberPerPatch-1; j++) {
	jFine = j - StartIndex[m][1] - 1;
	iFine = cellNumberPerPatch - 1;
	kFine = k - StartIndex[m][2] - 1;
	
	vCoarseGhostPoint(v);	
      }
    }


  //bottom
    k = StartIndex[m][2]+1;
    for (j=StartIndex[m][1]+1; j<=StartIndex[m][1]+cellNumberPerPatch-1; j++) {
      for (i=StartIndex[m][0]+1; i<=StartIndex[m][0]+cellNumberPerPatch; i++) {
	jFine = j - StartIndex[m][1] - 1;
	kFine = 0;
	iFine = i - StartIndex[m][0] - 1;
	
	vCoarseGhostPoint(v);	
      }
    }

  //top
    k = StartIndex[m][2]+cellNumberPerPatch;
    for (j=StartIndex[m][1]+1; j<=StartIndex[m][1]+cellNumberPerPatch-1; j++) {
      for (i=StartIndex[m][0]+1; i<=StartIndex[m][0]+cellNumberPerPatch; i++) {
	jFine = j - StartIndex[m][1] - 1;
	kFine = cellNumberPerPatch - 1;
	iFine = i - StartIndex[m][0] - 1;

	vCoarseGhostPoint(v);	
      }
    }


    ///////////////////////////////////////////////
    //
    //        for w
    //    
    //////////////////////////////////////////////



    //bottom
    k = StartIndex[m][2] + 1;
    for (i=StartIndex[m][0]+1; i<=StartIndex[m][0]+cellNumberPerPatch; i++) {
      for (j=StartIndex[m][1]+1; j<=StartIndex[m][1]+cellNumberPerPatch; j++) {
	kFine = 0;
	jFine = j - StartIndex[m][1] - 1;
	iFine = i - StartIndex[m][0] - 1;
	
	wCoarseGhostPoint(w);
      }
    }

  //top
    k = StartIndex[m][2]+cellNumberPerPatch-1;
    for (i=StartIndex[m][0]+1; i<=StartIndex[m][0]+cellNumberPerPatch; i++) {
      for (j=StartIndex[m][1]+1; j<=StartIndex[m][1]+cellNumberPerPatch; j++) {
	kFine = cellNumberPerPatch - 2;
	jFine = j - StartIndex[m][1] - 1;
	iFine = i - StartIndex[m][0] - 1;
	
	wCoarseGhostPoint(w);
      }
    }


  //south
    j = StartIndex[m][1]+1;
    for (k=StartIndex[m][2]+1; k<=StartIndex[m][2]+cellNumberPerPatch-1; k++) {
      for (i=StartIndex[m][0]+1; i<=StartIndex[m][0]+cellNumberPerPatch; i++) {
	kFine = k - StartIndex[m][2] - 1;
	jFine = 0;
	iFine = i - StartIndex[m][0] - 1;
	
	wCoarseGhostPoint(w);
      }
    }

  //north
    j = StartIndex[m][1]+cellNumberPerPatch;
    for (k=StartIndex[m][2]+1; k<=StartIndex[m][2]+cellNumberPerPatch; k++) {
      for (i=StartIndex[m][0]+1; i<=StartIndex[m][0]+cellNumberPerPatch-1; i++) {
	kFine = k - StartIndex[m][2] - 1;
	jFine = cellNumberPerPatch - 1;
	iFine = i - StartIndex[m][0] - 1;
	
	wCoarseGhostPoint(w);
      }
    }


  //west
    i = StartIndex[m][0]+1;
    for (j=StartIndex[m][1]+1; j<=StartIndex[m][1]+cellNumberPerPatch; j++) {
      for (k=StartIndex[m][2]+1; k<=StartIndex[m][2]+cellNumberPerPatch-1; k++) {
	kFine = k - StartIndex[m][2] - 1;
	iFine = 0;
	jFine = j - StartIndex[m][1] - 1;
	
	wCoarseGhostPoint(w);
      }
    }

  //east
    i = StartIndex[m][0]+cellNumberPerPatch;
    for (j=StartIndex[m][1]+1; j<=StartIndex[m][1]+cellNumberPerPatch; j++) {
      for (k=StartIndex[m][2]+1; k<=StartIndex[m][2]+cellNumberPerPatch-1; k++) {
	kFine = k - StartIndex[m][2] - 1;
	iFine = cellNumberPerPatch - 1;
	jFine = j - StartIndex[m][1] - 1;
	
	wCoarseGhostPoint(w);
      }
    }


  } // end for m
 
}


void ghostPoint::uCoarseGhostPoint(real ****u){
	//approach one
  u[0][i][j][k] = ( u[m][4+2*iFine][2+2*jFine][2+2*kFine]
		    + u[m][4+2*iFine][3+2*jFine][2+2*kFine]
		    + u[m][4+2*iFine][2+2*jFine][3+2*kFine]
		    + u[m][4+2*iFine][3+2*jFine][3+2*kFine]
		    ) / 4.0;   
}

void ghostPoint::vCoarseGhostPoint(real ****v){
  //approach one
  v[0][i][j][k] = ( v[m][2+2*iFine][4+2*jFine][2+2*kFine]
		    + v[m][3+2*iFine][4+2*jFine][2+2*kFine]
		    + v[m][2+2*iFine][4+2*jFine][3+2*kFine]
		    + v[m][3+2*iFine][4+2*jFine][3+2*kFine]
		    ) / 4.0; 
}

void ghostPoint::wCoarseGhostPoint(real ****w){
  //approach one
  w[0][i][j][k] = ( w[m][2+2*iFine][2+2*jFine][4+2*kFine]
		    + w[m][2+2*iFine][3+2*jFine][4+2*kFine]
		    + w[m][3+2*iFine][2+2*jFine][4+2*kFine]
		    + w[m][3+2*iFine][3+2*jFine][4+2*kFine]
		    ) / 4.0; 
}



void ghostPoint::uFineGhostPoint(real iy, // 0 for odd; 1 for even in y direction
				 real iz, // 0 for odd; 1 for even in z direction
				 real ****u){
  
  u[m][i][j][k] = ( pow(3.0, fabs(iy) + fabs(iz))
		    * u[0][iCoarse][jCoarse][kCoarse]     //SB
		    + pow(3.0, fabs(iy - 1) + fabs(iz))
		    * u[0][iCoarse][jCoarse+1][kCoarse]   //NB
		    + pow(3.0, fabs(iy) + fabs(iz - 1))
		    * u[0][iCoarse][jCoarse][kCoarse+1]   //ST
		    + pow(3.0, fabs(iy - 1) + fabs(iz - 1))
		    * u[0][iCoarse][jCoarse+1][kCoarse+1] //NT
		    ) / 16.0; 
}

void ghostPoint::vFineGhostPoint(real ix, // 0 for odd; 1 for even in x direction
				 real iz, // 0 for odd; 1 for even in z direction
				 real ****v){
  
  v[m][i][j][k] = ( pow(3.0, fabs(ix) + fabs(iz))
		    * v[0][iCoarse][jCoarse][kCoarse]     //WB
		    + pow(3.0, fabs(ix - 1) + fabs(iz))
		    * v[0][iCoarse+1][jCoarse][kCoarse]   //EB
		    + pow(3.0, fabs(ix) + fabs(iz - 1))
		    * v[0][iCoarse][jCoarse][kCoarse+1]   //WT
		    + pow(3.0, fabs(ix - 1) + fabs(iz - 1))
		    * v[0][iCoarse+1][jCoarse][kCoarse+1] //ET
		    ) / 16.0; 
}

void ghostPoint::wFineGhostPoint(real ix, // 0 for odd; 1 for even in x direction
				 real iy, // 0 for odd; 1 for even in y direction
				 real ****w){
  
  w[m][i][j][k] = ( pow(3.0, fabs(ix) + fabs(iy))
		    * w[0][iCoarse][jCoarse][kCoarse]     //WB
		    + pow(3.0, fabs(ix - 1) + fabs(iy))
		    * w[0][iCoarse+1][jCoarse][kCoarse]   //EB
		    + pow(3.0, fabs(ix) + fabs(iy - 1))
		    * w[0][iCoarse][jCoarse+1][kCoarse]   //WT
		    + pow(3.0, fabs(ix - 1) + fabs(iy - 1))
		    * w[0][iCoarse+1][jCoarse+1][kCoarse] //ET
		    ) / 16.0; 
}

ghostPoint::~ghostPoint(){
}
