#ifndef GHOSTPOINT_H
#define GHOSTPOINT_H

/** \file ghostPoint.cpp
    \brief Interpolate the ghost velocity points at the interface of two meshes.

    This class interpolates the ghost velocity points at the interface of two meshes.

*/

#include <iostream>
#include <string>
#include <cmath>
using namespace std;

#ifdef SINGLE
#define real float
#else
#define real double
#endif

/** 
    \brief Interpolate the ghost velocity points at the interface of two meshes.

    This class interpolates the ghost velocity points at the interface of two meshes.

*/

class ghostPoint {

 public:
  /**
     constructor
   */
  ghostPoint();

/** 
    \brief Interpolate the ghost velocity points at the interface of two meshes.

    This class interpolates the ghost velocity points at the interface of two meshes.

*/
  void initialize(int numberOfPatches, /**< number of patches */
		  real t,  /**<  time */
		  real dt, /**<  time increment */
		  int cellNumberPerPatch, /**<  cell numbers per patch surrounding the sphere*/
		  int ** StartIndex,    /**< used to put the refined mesh */
		  int ** pointNumberMeshU, /**< mesh points for u */
		  int ** pointNumberMeshV, /**< mesh points for v */
		  int ** pointNumberMeshW, /**< mesh points for w */
		  int dimension, /**<  dimension */
		  real ***xU, /**< coordinates of u */
		  real ***xV, /**< coordinates of v */
		  real ***xW, /**< coordinates of w */
		  real *h, /**< space step */
		  real ****u, /**< velocity u */
		  real ****v, /**< velocity v */
		  real ****w  /**< velocity w */
		  );

  /**
     destructor
   */
  ~ghostPoint();
  
 private: 

  void uCoarseGhostPoint(real ****u);

  void vCoarseGhostPoint(real ****v);

  void wCoarseGhostPoint(real ****w);
  
  void uFineGhostPoint(real iy, // 0 for odd; 1 for even in y direction
		       real iz, // 0 for odd; 1 for even in z direction
		       real ****u);

  void vFineGhostPoint(real ix, // 0 for odd; 1 for even in x direction
		       real iz, // 0 for odd; 1 for even in z direction
		       real ****v);

  void wFineGhostPoint(real ix, // 0 for odd; 1 for even in x direction
		       real iy, // 0 for odd; 1 for even in y direction
		       real ****w);
    
  int nPatch; // each patch carries a square fine mesh
  int nMeshes;   // including the coarse mesh
  // the coarse mesh is the zeroth mesh and fine meshes are from 1st to the last
  int nCellPerPatch; // cell number in the coarse mesh that a patch uses

  int dim;


  // iRHS for index in matrix A = totalUnknowns x totalUnknowns
  // jRHS
  int i, j, k;

  int iFine, jFine, kFine;
  int iCoarse, jCoarse, kCoarse;

  // m for mesh iteration: meshes for patchs and coarse grid
  int m;


};

#endif
