#ifndef INITCOND_H
#define INITCOND_H

/** \file initCond.cpp
    \brief Set initial conditions.

    This class sets the initial conditions for velocity.

*/

#include <iostream>
#include <string>
#include <cmath>
using namespace std;

#ifdef SINGLE
#define real float
#else
#define real double
#endif

/** 
    \brief Set initial conditions.

    This class sets the initial conditions for velocity.

*/
class initCond {

 public:
  /**
     constructor
   */
  initCond();

/** 
    \brief Set initial conditions.

    This class sets the initial conditions for velocity.

*/
  void initialize(int nMeshes, /**< number of meshes */
		  int ** pointNumberMeshU, /**< mesh points for u */
		  real ***xU, /**< coordinates of u */
		  int ** pointNumberMeshV, /**< mesh points for v */
		  real ***xV, /**< coordinates of v */
		  int ** pointNumberMeshW, /**< mesh points for w */
		  real ***xW, /**< coordinates of w */
		  int ** pointNumberMeshP, /**< mesh points for p */
		  real ***xP, /**< coordinates of p */
		  real ****u, /**< velocity u */
		  real ****v, /**< velocity v */
		  real ****w, /**< velocity w */
		  real ****p  /**< velocity p */
		  ); 

  /**
     destructor
   */
  ~initCond();
  
 private: 


  // iRHS for index in matrix A = totalUnknowns x totalUnknowns
  // jRHS
  int i, j, k;


  // m for mesh iteration: meshes for patchs and coarse grid
  int m;

  // nx and ny for wavenumber in x and y directions
  int nx, ny, nz;


};

#endif
