#include "initStage.h"


initStage::initStage(){
}

void initStage::solve(int nPatch,
		      real &t,
		      real &dt,
		      real Re,
		      int nTruncate,
		      int cellNumberPerPatch, 
		      int ** StartIndex, 
		      real *xParticleCenter,  // coordinates of particle center
		      real *yParticleCenter, 
		      real *zParticleCenter, 
		      int ** pointNumberMeshU, 
		      int ** pointNumberMeshV, 
		      int ** pointNumberMeshW, 
		      int ** pointNumberMeshP, 
		      int dim,
		      real ***xU, 
		      real ***xV, 
		      real ***xW, 
		      real ***xP, 
		      real *h,
		      real ****u,
		      int ****uUnknowns,
		      real ****v,
		      int ****vUnknowns,
		      real ****w,
		      int ****wUnknowns,
		      real ****p,
		      real ****uAux,
		      real ****vAux,
		      real ****wAux,
		      real ****fp, 
		      real ****exactP, 
		      int ****pUnknowns,
		      int ****uCagePoint,
		      int ****vCagePoint,
		      int ****wCagePoint,
		      int ****pCagePoint,
		      int **nCell,
		      PetscLinearSolver &pSolver,
		      int &zeroInitialGuess){
  

  NumberOfMeshes = nPatch + 1;

  uIter = allocate4dRealArray(NumberOfMeshes, pointNumberMeshU);
  vIter = allocate4dRealArray(NumberOfMeshes, pointNumberMeshV);
  wIter = allocate4dRealArray(NumberOfMeshes, pointNumberMeshW);

  //copy to arrays for the iteration of cage points

  //use the initial field as a guess

  for (m=0; m<=nPatch; m++) {
    for (k=0; k<pointNumberMeshU[m][2]; k++) {
      for (j=0; j<pointNumberMeshU[m][1]; j++) {
	for (i=0; i<pointNumberMeshU[m][0]; i++) {
	  uIter[m][i][j][k] = u[m][i][j][k];
	}
      }
    }
  }

  for (m=0; m<=nPatch; m++) {
    for (k=0; k<pointNumberMeshV[m][2]; k++) {
      for (j=0; j<pointNumberMeshV[m][1]; j++) {
	for (i=0; i<pointNumberMeshV[m][0]; i++) {
	  vIter[m][i][j][k] = v[m][i][j][k];
	}
      }
    }
  }  

  for (m=0; m<=nPatch; m++) {
    for (k=0; k<pointNumberMeshW[m][2]; k++) {
      for (j=0; j<pointNumberMeshW[m][1]; j++) {
	for (i=0; i<pointNumberMeshW[m][0]; i++) {
	  wIter[m][i][j][k] = w[m][i][j][k];
	}
      }
    }
  }  

  int totalIter = 1;
  for (iter=1; iter<=totalIter; iter++) {
 
    cout << "initial stage: " <<iter<<endl;   
    // set internal velocity boundary conditions
    cageInit cObj;
    cObj.solve(nPatch,
	       cellNumberPerPatch,
	       dim,
	       // for u
	       uIter,
	       pointNumberMeshU,
	       uCagePoint,
	       // for v
	       vIter,
	       pointNumberMeshV,
	       vCagePoint,
	       // for w
	       wIter,
	       pointNumberMeshW,
	       wCagePoint);

  setDt sObj;
  sObj.initialize(nPatch,
		  dt,
		  Re,
		  pointNumberMeshU, 
		  pointNumberMeshV, 
		  pointNumberMeshW, 
		  h,
		  uIter,
		  uUnknowns,
		  vIter,
		  vUnknowns,
		  wIter,
		  wUnknowns);


    projectionMethod pObj;
    pObj.solve(nPatch,
	       t,
	       dt,
	       Re,
	       cellNumberPerPatch, 
	       StartIndex, 
	       pointNumberMeshU, 
	       pointNumberMeshV, 
	       pointNumberMeshW, 
	       pointNumberMeshP, 
	       dim,
	       xU, 
	       xV, 
	       xW, 
	       xP, 
	       h,
	       uIter,
	       uIter,
	       uCagePoint,
	       vIter,
	       vIter,
	       vCagePoint,
	       wIter,
	       wIter,
	       wCagePoint,
	       p,
	       uAux,
	       vAux,
	       wAux,
	       fp, 
	       exactP, 
	       pUnknowns,
	       pCagePoint,
	       pSolver,
	       zeroInitialGuess);
    
    //    t = t + dt;
    
  }

  /*    
  for (m=1; m<=nPatch; m++) {
    cout <<"============p==========="<<endl;
    for (j=0; j<pointNumberMeshP[m][1]; j++) {
      for (i=0; i<pointNumberMeshP[m][0]; i++) {
	cout << p[m][i][j] << " ";
      }
      cout << endl;
    }
  }        
*/    


  for (m=0; m<=nPatch; m++) {
    //       cout <<"============u==========="<<endl;
    for (k=0; k<pointNumberMeshU[m][2]; k++) {
      for (j=0; j<pointNumberMeshU[m][1]; j++) {
	for (i=0; i<pointNumberMeshU[m][0]; i++) {
	  u[m][i][j][k] = uIter[m][i][j][k];
	  //		cout << u[m][i][j] << " ";
	}
	//      cout << endl;
      }
    }
  }



  for (m=0; m<=nPatch; m++) {
    for (k=0; k<pointNumberMeshV[m][2]; k++) {
      for (j=0; j<pointNumberMeshV[m][1]; j++) {
	for (i=0; i<pointNumberMeshV[m][0]; i++) {
	  v[m][i][j][k] = vIter[m][i][j][k];
	}
      }
    }
  }

  for (m=0; m<=nPatch; m++) {
    //       cout <<"============u==========="<<endl;
    for (k=0; k<pointNumberMeshW[m][2]; k++) {
      for (j=0; j<pointNumberMeshW[m][1]; j++) {
	for (i=0; i<pointNumberMeshW[m][0]; i++) {
	  w[m][i][j][k] = wIter[m][i][j][k];
	  //		cout << u[m][i][j] << " ";
	}
	//      cout << endl;
      }
    }
  }



  delete4dArray(uIter, NumberOfMeshes, pointNumberMeshU);
  delete4dArray(vIter, NumberOfMeshes, pointNumberMeshV);
  delete4dArray(wIter, NumberOfMeshes, pointNumberMeshW);

}

real **** initStage::allocate4dRealArray(int nMeshes, int **pointNumberMesh){
  real ****x = new real ***[nMeshes];
  for (m=0; m<nMeshes; m++) {
    x[m] = new real **[pointNumberMesh[m][0]];
    for (i=0; i<pointNumberMesh[m][0]; i++) {
      x[m][i] = new real*[pointNumberMesh[m][1]];
      for (j=0; j<pointNumberMesh[m][1]; j++) {
	x[m][i][j] = new real[pointNumberMesh[m][2]];
      }
    }
  }
  return x;
}

void initStage::delete4dArray(real ****x, int nMeshes, int **pointNumberMesh){
  for (m=0; m<nMeshes; m++) {
    for (i=0; i<pointNumberMesh[m][0]; i++) {
      for (j=0; j<pointNumberMesh[m][1]; j++) {
	delete [] x[m][i][j];
      }
      delete [] x[m][i];
    }
    delete [] x[m];
  }
  delete [] x;
}



initStage::~initStage(){
}
