#ifndef INITSTAGE_H
#define INITSTAGE_H

/** \file initStage.cpp
    \brief Calculate the initial stage using the velcity at cage points from cageInit.

    This class is similar to NSSolver. except that the velocity at cage points is from cageInit rather than from cage.

*/

#include "projectionMethod.h"
#include "cageInit.h"
#include "vorticity.h"
#include "setDt.h"
#include <string>
#include <iostream>

#ifdef SINGLE
#define real float
#else
#define real double
#endif

/** 
    \brief Calculate the initial stage using the velcity at cage points from cageInit.

    This class is similar to NSSolver. except that the velocity at cage points is from cageInit rather than from cage.

*/
class initStage {

 public:

  /**
     constructor
   */
  initStage();

/** 
    \brief Calculate the initial stage using the velcity at cage points from cageInit.

    This class is similar to NSSolver. except that the velocity at cage points is from cageInit rather than from cage.

*/
  void solve(int nPatch, /**< number of patches */
	     real &t, /**<  time */
	     real &dt, /**<  time increment */
	     real Re, /**<  Reynolds number */
	     int nTruncate, /**<  the order for truncated spectral analytical solution*/
	     int cellNumberPerPatch,  /**<  cell numbers per patch surrounding the sphere*/
	     int ** StartIndex, /**< used to put the refined mesh */
	     real *xParticleCenter,  /**< x coordinate of particle center */
	     real *yParticleCenter, /**< y coordinate of particle center */
	     real *zParticleCenter, /**< z coordinate of particle center */
	     int ** pointNumberMeshU, /**< mesh points for u */
	     int ** pointNumberMeshV, /**< mesh points for v */
	     int ** pointNumberMeshW, /**< mesh points for w */
	     int ** pointNumberMeshP, /**< mesh points for p */
	     int dim, /**<  dimension */
	     real ***xU, /**< coordinates of u  */
	     real ***xV, /**< coordinates of v  */
	     real ***xW, /**< coordinates of w  */
	     real ***xP, /**< coordinates of p  */
	     real *h, /**< space step  */
	     real ****u, /**< velocity u */
	     int ****uUnknowns, /**< mesh points for u */
	     real ****v, /**< velocity v */
	     int ****vUnknowns, /**< mesh points for v */
	     real ****w, /**< velocity w */
	     int ****wUnknowns, /**< mesh points for w */
	     real ****p, /**<  pressure */
	     real ****uAux, /**< predicted u */
	     real ****vAux, /**< predicted v */
	     real ****wAux, /**< predicted w */
	     real ****fp,  /**< extra force */
	     real ****exactP,  /**< exact pressure */
	     int ****pUnknowns, /**< mesh points for p */
	     int ****uCagePoint, /**< cage points for u */
	     int ****vCagePoint, /**< cage points for v */
	     int ****wCagePoint, /**< cage points for w */
	     int ****pCagePoint, /**< cage points for p */
	     int **nCell, /**< number of cells */
	     PetscLinearSolver &pSolver, /**< PETSC linear solver */
	     int &zeroInitialGuess /**< if use zero as the initial guess for pressure */
	     );

  /**
     destructor
   */
  ~initStage();

 private: 

  real **** allocate4dRealArray(int nMeshes, int **pointNumberMesh);
  void delete4dArray(real ****x, int nMeshes, int **pointNumberMesh);

  int m, i, j, k, iter;

  int NumberOfMeshes;

  real ****uIter; // solution corresponding to uUnknowns
  real ****vIter; // solution corresponding to vUnknowns
  real ****wIter; // solution corresponding to wUnknowns

};

#endif
