c******************************************************************************
c     Calculates the associated Legendre polynomials of positve
c                   degree and order
c     P_l^m(x) = (1-x^2)^(m/2) d^m/(dx)^m P_l(x) --- Ferrer expression         
c*****************************************************************************
      subroutine alegp(lmax,mmax,x,p,ldp)
      implicit real*8 (a-h,o-z)
      integer lmax,mmax,ldp
      real*8 x
      real*8 p(0:ldp,0:mmax)
      integer l,m
      do m=0,mmax
         do l=0,m-1
            p(l,m)=0
         enddo
         p(m,m)=1
         if(m.gt.0) then
            somx2=sqrt((1-x)*(1+x))
            fact=1
            do i=1,m
               p(m,m)=p(m,m)*fact*somx2
               fact=fact+2
            enddo
         endif
         if(lmax.ge.m+1) p(m+1,m)=x*(2*m+1)*p(m,m)
         do l=m+2,lmax
            p(l,m)=(x*(2*l-1)*p(l-1,m)-(l+m-1)*p(l-2,m))/(l-m)
         enddo
      enddo
      return
      end
c******************************************************************************
c     Calculates the associated Legendre polynomials of positve
c                   degree and order
c     P_n^m, P_n^m/sin(theta), -sin(theta)dP_n^m/dx
c     pl = P_n^m(cos t)               m=0,1,2,... n=0,1,2,...
c     ps = P_n^m(cos t) / sin t       m=1,2,3,... n=0,1,2,...
c     pd = - sin t (d/dx) [P_n^m(x)]  m=0,1,2,... n=0,1,2,...
c*****************************************************************************
      subroutine alegpsd(lmax,mmax,x,pl,ps,pd,ldp)
      implicit real*8 (a-h,o-z)
      integer lmax,mmax,ldp
      real*8 x
      real*8 pl(0:ldp,0:mmax)
      real*8 ps(0:ldp,0:mmax)
      real*8 pd(0:ldp,0:mmax)
      integer l,m
      do m=0,mmax
         do l=0,m-1
            pl(l,m)=0.0
            ps(l,m)=0.0
            pd(l,m)=0.0
         enddo
         pl(m,m)=1
         ps(m,m)=1
         if(m.gt.0) then
            somx2=sqrt((1-x)*(1+x))
            fact=1
            do i=1,m
               pl(m,m)=pl(m,m)*fact*somx2
               fact=fact+2
            enddo
            fact=1
            ps(1,1)=ps(1,1)*fact
            fact=fact+2
            do i=2,m
               ps(m,m)=ps(m,m)*fact*somx2
               fact=fact+2
            enddo
         endif
         if(lmax.ge.m+1)then
            pl(m+1,m)=x*(2*m+1)*pl(m,m)
            ps(m+1,m)=x*(2*m+1)*ps(m,m)
         endif
         do l=m+2,lmax
            pl(l,m)=(x*(2*l-1)*pl(l-1,m)-(l+m-1)*pl(l-2,m))/(l-m)
            ps(l,m)=(x*(2*l-1)*ps(l-1,m)-(l+m-1)*ps(l-2,m))/(l-m)
         enddo

         if(m.ge.1)then
            pd(m,m)=m*x*ps(m,m)
            do l=m+1,lmax
               pd(l,m)=l*x*ps(l,m)-(l+m)*ps(l-1,m)
            enddo
         endif
      enddo
      if(mmax.ge.1)then
         do l=0,lmax
            pd(l,0)=-pl(l,1)
         enddo
      else
         pd(0,0)=0
         if(lmax.gt.1) then
            pd(1,0)=1
         endif
         do l=2,lmax
            pd(l,0)=pd(l-2,0)+(2*l-1)*pl(l-1,0)
         enddo
         somx2=sqrt((1-x)*(1+x))
         do l=0,lmax
            pd(l,0)=-pd(l,0)*somx2
         enddo
      endif
      return
      end

