// i and j are consistent with the physical domain
// i.e., same i means same x coordinate
//       same j means same y coordinate
// discretization stencil is:
//              (i,j+1)
//     (i-1,j)  (i, j )   (i+1,j)
//              (i,j-1)

/** \file main.cpp
    \brief start the simulation

    This function first initialize PETSC, then call the class flowSolver to do the actual simulation, and finally exit PETSC.

*/

#include "flowSolver.h"
#include "PetscLinearSolver.h"
#include "constant.h"
#include <cmath>
#include <fstream>

/** 
    \brief start the simulation

    This function first initialize PETSC, then call the class flowSolver to do the actual simulation, and finally exit PETSC.

    @param argc number of arguments in the command line
    @param args arguments in the command line
*/

int main(int argc,char **args)
{

  // don't remove
  // initialize PETSc
  static char help[] = "Physalis 2d with KSP.\n\n";
  PetscInitialize(&argc,&args,(char *)0,help);


  //solve the flow field
  flowSolver solver;
  solver.solve();


  // don't remove
  // close PETSc

  //     Always call PetscFinalize() before exiting a program.  This routine
  //       - finalizes the PETSc libraries as well as MPI
  //       - provides summary and diagnostic information if certain runtime
  //         options are chosen (e.g., -log_summary).
  PetscErrorCode ierr;
  ierr = PetscFinalize();CHKERRQ(ierr);


  return 0;
}
