#ifndef OUTPUTDATA_H
#define OUTPUTDATA_H

#include <iostream>
#include <fstream>
#include <sstream>
#include <cmath>
using namespace std;

#ifdef SINGLE
#define real float
#else
#define real double
#endif

/** \file outputData.cpp
    \brief output velocity and pressure to files.

    Output velocity and pressure to files for the postprocessing.  
*/

/** 
    \brief output velocity and pressure to files.

    Output velocity and pressure to files for the postprocessing. The values are for points of center of all fine meshes in the collocation sense.
*/
class outputData {

 public:
  /**
     constructor
   */
  outputData();

/** 
    \brief output velocity and pressure to files.

     Output velocity and pressure to files for the postprocessing. The values are for points of center of all fine meshes in the collocation sense.
*/
  void initialize(int k, /**<  time  step */
		  int NumberOfMeshes, /**< number of meshes */
		  int cellNumberPerPatch,/**<  cell numbers per patch surrounding the sphere*/
		  int ** StartIndex, /**< used to put the refined mesh */
		  int ** pointNumberMeshU, /**< mesh points for u */
		  int ** pointNumberMeshV, /**< mesh points for v */
		  int ** pointNumberMeshW, /**< mesh points for w */
		  int ** pointNumberMeshP, /**< mesh points for p */
		  real ***xU, /**< coordinates of u  */
		  real ***xV, /**< coordinates of v  */
		  real ***xW, /**< coordinates of w  */
		  real ***xP, /**< coordinates of p  */
		  real ****u, /**< velocity u */
		  real ****v,/**< velocity v */
		  real ****w,/**< velocity w */
		  real ****p /**< velocity p */
		  );

  /**
     destructor
   */
  ~outputData();
  
 private: 

  void outputU(int k,
	       int NumberOfMeshes,
	       int cellNumberPerPatch,
	       int ** StartIndex, 
	       int ** pointNumberMeshU, 
	       int ** pointNumberMeshP, 
	       real ***xU, 
	       real ***xP, 
	       real ****u);
  void outputV(int tStep,
	       int NumberOfMeshes,
	       int cellNumberPerPatch,
	       int ** StartIndex, 
	       int ** pointNumberMeshV, 
	       int ** pointNumberMeshP, 
	       real ***xV, 
	       real ***xP, 
	       real ****v);
  void outputW(int tStep,
	       int NumberOfMeshes,
	       int cellNumberPerPatch,
	       int ** StartIndex, 
	       int ** pointNumberMeshW, 
	       int ** pointNumberMeshP, 
	       real ***xW, 
	       real ***xP, 
	       real ****w);
  void outputP(int k,
	       int NumberOfMeshes,
	       int cellNumberPerPatch,
	       int ** StartIndex, 
	       int ** pointNumberMeshP, 
	       real ***xP, 
	       real ****p);

  real **** allocate4dRealArrayOutput(int nMeshes, int **pointNumberMesh);
  void delete4dArrayOutput(real ****x, int nMeshes, int **pointNumberMesh);
  string iToString(int k);
  void coarseToCenter(real ix, // 0 for odd; 1 for even in x direction
		      real iy, // 0 for odd; 1 for even in y direction
		      real iz, // 0 for odd; 1 for even in z direction
		      real ****u,
		      real ****uOutput);  

  string filenameU ;
  string filenameV ;
  string filenameW ;
  string filenameP ;
  
  ofstream ofU ;
  ofstream ofV ;
  ofstream ofW ;
  ofstream ofP ;
  

  // iRHS for index in matrix A = totalUnknowns x totalUnknowns
  // jRHS
  int i, j,k, m;

  int iCenter, jCenter,kCenter;


};

#endif
