#ifndef PROJECTIONMETHOD_H
#define PROJECTIONMETHOD_H

/** \file projectionMethod.cpp
    \brief the projection method solver 

    This class is the acutal projection method Solver. It call uStar to calculate the predicted u*, call divUStar to calculate the divergence of u*, calls pressure to calculate pressure, and calls ghostPoint to do interpolation for the velocity. It call call the cage to set inner boundary conditions associated with spheres.

*/

#include <iostream>
#include <string>
#include "uStar.h"
#include "divUStar.h"
#include "pressure.h"
#include "uNew.h"
using namespace std;

#ifdef SINGLE
#define real float
#else
#define real double
#endif

/** 
    \brief the projection method solver 

    This class is the acutal projection method Solver. It call uStar to calculate the predicted u*, call divUStar to calculate the divergence of u*, calls pressure to calculate pressure, and calls ghostPoint to do interpolation for the velocity. It call call the cage to set inner boundary conditions associated with spheres.

*/
class projectionMethod {

 public:

  /**
     constructor
   */
  projectionMethod();

/** 
    \brief the projection method solver 

    This class is the acutal projection method Solver. It call uStar to calculate the predicted u*, call divUStar to calculate the divergence of u*, calls pressure to calculate pressure, and calls ghostPoint to do interpolation for the velocity. It call call the cage to set inner boundary conditions associated with spheres.

*/
  void solve(int nPatch,/**< number of patches */
	     real t,/**<  time */
	     real dt,/**<  time increment */
	     real Re,/**<  Reynolds number */
	     int cellNumberPerPatch, /**<  cell numbers per patch surrounding the sphere*/
	     int ** StartIndex, /**< used to put the refined mesh */
	     int ** pointNumberMeshU, /**< mesh points for u */
	     int ** pointNumberMeshV, /**< mesh points for v */
	     int ** pointNumberMeshW, /**< mesh points for w */
	     int ** pointNumberMeshP, /**< mesh points for p */
	     int dim,/**<  dimension */
	     real ***xU, /**< coordinates of u  */
	     real ***xV, /**< coordinates of v  */
	     real ***xW, /**< coordinates of w  */
	     real ***xP, /**< coordinates of p  */
	     real *h,/**< space step  */
	     real ****u, /**< velocity u */
	     real ****uIter,/**< velocity u in the iteration*/
	     int ****uCagePoint,/**< cage points for u */
	     real ****v,/**< velocity v */
	     real ****vIter,/**< velocity v in the iteration */
	     int ****vCagePoint,/**< cage points for v */
	     real ****w,/**< velocity w */
	     real ****wIter,/**< velocity w in the iteration */
	     int ****wCagePoint,/**< cage points for w */
	     real ****p,/**<  pressure */
	     real ****uAux,/**< predicted u */
	     real ****vAux,/**< predicted u */
	     real ****wAux,/**< predicted u */
	     real ****fp, /**< extra force */
	     real ****exactP, /**< exact pressure */
	     int ****pUnknowns,/**< mesh points for p */
	     int ****pCagePoint,/**< cage points for p */
	     PetscLinearSolver &pSolver,/**< PETSC linear solver */
	     int &zeroInitialGuess/**< if use zero as the initial guess for pressure */
	     );

  /**
     destructor
   */
  ~projectionMethod();

 private: 

  int m, i, j, k;

};

#endif
