#include "setDt.h"


setDt::setDt(){
}

void setDt::initialize(int numberOfPatches,
		       real & dt,
		       real ReynoldsNumber,
		       int ** pointNumberMeshU, 
		       int ** pointNumberMeshV, 
		       int ** pointNumberMeshW, 
		       real *h,
		       real ****u,
		       int ****uUnknowns,
		       real ****v,
		       int ****vUnknowns,
		       real ****w,
		       int ****wUnknowns)
{

  nPatch = numberOfPatches;
  nMeshes = numberOfPatches + 1;
  Re = ReynoldsNumber;

  int i_max, j_max, k_max;

  // set dt such that
  // dt <= Re * dx^2 / 6
  // dt <= 4/ Re /(|U| + |V| + |W|)^2
  // dt <= C dx / U

  dt = 0.1;

  for (m=0; m<nMeshes; m++) {
    if (dt > Re * pow(h[m], 2) / 6.0 ) {
      dt = Re * pow(h[m], 2) / 6.0;  
    }
  }
  


  ///////////////////////////////////////////
  //
  //the coarse mesh
  //
  //////////////////////////////////////////
  for (m=0; m<nMeshes; m++) {
    
    real absUMax = 0.0;
    real absVMax = 0.0;
    real absWMax = 0.0;
    
    // find max u values
    for (k=1; k<pointNumberMeshU[m][2] - 1; k++) {
      for (j=1; j<pointNumberMeshU[m][1] - 1; j++) {
	for (i=1; i<pointNumberMeshU[m][0] - 1; i++) {
	  if (uUnknowns[m][i][j][k] == 1 
	      && fabs(u[m][i][j][k]) > absUMax){
	    absUMax = fabs(u[m][i][j][k]);
	  }
	}
      }
    }
    
    for (k=1; k<pointNumberMeshV[m][2] - 1; k++) {
      for (j=1; j<pointNumberMeshV[m][1] - 1; j++) {
	for (i=1; i<pointNumberMeshV[m][0] - 1; i++) {
	  if (vUnknowns[m][i][j][k] == 1 
	      && fabs(v[m][i][j][k]) > absVMax){
	    absVMax = fabs(v[m][i][j][k]);

	    if( m == 0) {
	      i_max = i;
	      j_max = j;
	      k_max = k;
	    }

	  }
	}
      }
    }
    
    for (k=1; k<pointNumberMeshW[m][2] - 1; k++) {
      for (j=1; j<pointNumberMeshW[m][1] - 1; j++) {
	for (i=1; i<pointNumberMeshW[m][0] - 1; i++) {
	  if (wUnknowns[m][i][j][k] == 1 
	      && fabs(w[m][i][j][k]) > absWMax){
	    absWMax = fabs(w[m][i][j][k]);
	  }
	}
      }
    }

    /*    
    if (dt * pow(absUMax + absVMax + absWMax, 2.0) * Re > 4.0){
      dt = 4.0 / Re /pow(absUMax + absVMax + absWMax, 2.0);
    }
    */

    
    if (dt * pow(absUMax, 2.0) * Re > 4.0){
      dt = 4.0 / Re /pow(absUMax, 2.0) * 0.9;
    }
    

    /*
    if (dt * pow(absVMax, 2.0) * Re > 4.0){
      dt = 4.0 / Re /pow(absVMax, 2.0) * 0.9;
    }
    */

    /*  
    if (m == 1){   
      cout << "          m= " << m <<endl;
      cout << "            absUMax = " << absUMax <<endl;
      cout << "            absVMax = " << absVMax <<endl;
      cout << "            absWMax = " << absWMax <<endl;
    }
    */
   
  }

  /*
  cout << endl;
  cout << endl;
  cout << "            i_max = " << i_max <<endl;
  cout << "            j_max = " << j_max <<endl;
  cout << "            k_max = " << k_max <<endl;
  */
    dt = dt * 0.99; // use a safty coefficient 0.99 for stability

    //  cout << "    maximum   dt = " << dt <<endl;

  // fixed dt
  //  dt = 2.5e-4;

  cout << "    time step dt = " << dt <<endl;
  cout <<endl;


}



setDt::~setDt(){
}
