#ifndef SETDT_H
#define SETDT_H

/** \file setDt.cpp
    \brief set time increment dt for stability.

    This class is used to set time increment dt for stability .

*/
#include <iostream>
#include <string>
#include <cmath>
using namespace std;

#ifdef SINGLE
#define real float
#else
#define real double
#endif

/** 
    \brief set time increment dt for stability.

    This class is used to set time increment dt for stability .

*/
class setDt {

 public:
  /**
     constructor
   */
  setDt();

  /** 
    \brief set time increment dt for stability.

    This class is used to set time increment dt for stability, which satisfies two equations from the textbook. The solver uses the standard explicit Euler scheme for time intergration and centeral difference scheme for spatial difference.

*/
  void initialize(int numberOfPatches,/**< number of patches */
		  real & dt,/**<  time increment */
		  real ReynoldsNumber,/**<  Reynolds number */
		  int ** pointNumberMeshU, /**< mesh points for u */
		  int ** pointNumberMeshV, /**< mesh points for v */
		  int ** pointNumberMeshW, /**< mesh points for w */
		  real *h,/**< space step  */
		  real ****u,/**< velocity u */
		  int ****uUnknowns,/**< cage points for u */
		  real ****v,/**< velocity v */
		  int ****vUnknowns,/**< cage points for v */
		  real ****w,/**< velocity w */
		  int ****wUnknowns/**< cage points for w */
		  );

  /**
     destructor
   */
  ~setDt();
  
 private: 


  int nPatch; // each patch carries a square fine mesh
  int nMeshes;   // including the coarse mesh
  // the coarse mesh is the zeroth mesh and fine meshes are from 1st to the last

  real Re; // Reynolds number based on the radius of a particle

  // iRHS for index in matrix A = totalUnknowns x totalUnknowns
  // jRHS
  int i, j, k;


  // m for mesh iteration: meshes for patchs and coarse grid
  int m;



};

#endif
