#include "uNew.h"


uNew::uNew(){
}

void uNew::initialize(int numberOfPatches,
		      real t,
		      real dt,
		      int cellNumberPerPatch, 
		      int ** StartIndex, 
		      int ** pointNumberMeshU, 
		      int ** pointNumberMeshV, 
		      int ** pointNumberMeshW, 
		      int ** pointNumberMeshP, 
		      int dimension,
		      real ***xU, 
		      real ***xV, 
		      real ***xW, 
		      real *h,
		      real ****u,
		      int ****uCagePoint,
		      real ****v,
		      int ****vCagePoint,
		      real ****w,
		      int ****wCagePoint,
		      real ****uAux,
		      real ****vAux,
		      real ****wAux,
		      real ****p)
{

  nPatch = numberOfPatches;
  nMeshes = numberOfPatches + 1;
  nCellPerPatch = cellNumberPerPatch;
  dim = dimension;

  ////////////////////////////////////
  //
  //interior points of u
  //note the symmetry of j and k
  //
  ////////////////////////////////////

  //the coarse mesh
  for (k=1; k<pointNumberMeshU[0][2] - 1; k++) {
    for (j=1; j<pointNumberMeshU[0][1] - 1; j++) {
      for (i=1; i<pointNumberMeshU[0][0] - 1; i++) {
	u[0][i][j][k] = uAux[0][i][j][k] 
	  - dt * (p[0][i][j-1][k-1] - p[0][i-1][j-1][k-1]) / h[0]; 
      }
    }
  }

  //partilces
  for (m=1; m<=nPatch; m++) {
    for (k=1; k<pointNumberMeshU[m][2]-1; k++) {
      for (j=1; j<pointNumberMeshU[m][1]-1; j++) {
	for (i=1; i<pointNumberMeshU[m][0]-1; i++) {

	  if (uCagePoint[m][i][j][k] == 0){ // keep inner boundary for particles
	    u[m][i][j][k] = uAux[m][i][j][k]
	      - dt * (p[m][i][j][k] - p[m][i-1][j][k]) / h[m]; 
	  }

	}
      }
    }
  }


  //interior points of v
  //note the symmetry of i and k

  //the coarse mesh
  for (k=1; k<pointNumberMeshV[0][2] - 1; k++) {
    for (j=1; j<pointNumberMeshV[0][1] - 1; j++) {
      for (i=1; i<pointNumberMeshV[0][0] - 1; i++) {
	v[0][i][j][k] = vAux[0][i][j][k] 
	  - dt * (p[0][i-1][j][k-1] - p[0][i-1][j-1][k-1]) / h[0];  
      }
    }
  }

  //partilces
  for (m=1; m<=nPatch; m++) {
    for (k=1; k<pointNumberMeshV[m][2]-1; k++) {
      for (j=1; j<pointNumberMeshV[m][1]-1; j++) {
	for (i=1; i<pointNumberMeshV[m][0]-1; i++) {

	  if (vCagePoint[m][i][j][k] == 0) { // keep inner boundary for particles
	    v[m][i][j][k] = vAux[m][i][j][k]  
	      - dt * (p[m][i][j][k] - p[m][i][j-1][k]) / h[m];  
	  }

	}
      }
    }
  }

  //interior points of w
  //note the symmetry of i and j

  //the coarse mesh
  for (k=1; k<pointNumberMeshW[0][2] - 1; k++) {
    for (j=1; j<pointNumberMeshW[0][1] - 1; j++) {
      for (i=1; i<pointNumberMeshW[0][0] - 1; i++) {
	w[0][i][j][k] = wAux[0][i][j][k] 
	  - dt * (p[0][i-1][j-1][k] - p[0][i-1][j-1][k-1]) / h[0];  
      }
    }
  }


  //partilces
  for (m=1; m<=nPatch; m++) {
    for (k=1; k<pointNumberMeshW[m][2]-1; k++) {
      for (j=1; j<pointNumberMeshW[m][1]-1; j++) {
	for (i=1; i<pointNumberMeshW[m][0]-1; i++) {
	  
	  if (wCagePoint[m][i][j][k] == 0) {  // keep inner boundary for particles
	    w[m][i][j][k] = wAux[m][i][j][k]  
	      - dt * (p[m][i][j][k] - p[m][i][j][k-1]) / h[m];
	  }

	}
      }
    }
  }

  ////////////////////////////////////
  //
  //interface points 
  //
  ////////////////////////////////////


  ghostPoint g;
  g.initialize(nPatch,
	       t,
	       dt,
	       cellNumberPerPatch, 
	       StartIndex, 
	       pointNumberMeshU, 
	       pointNumberMeshV, 
	       pointNumberMeshW, 
	       dim,
	       xU, 
	       xV, 
	       xW, 
	       h,
	       u,
	       v,
	       w);

  ////////////////////////////////////
  //
  // outer boundary: periodic
  //
  ////////////////////////////////////


  /*    
  i = 0;
  for (k=1; k<pointNumberMeshU[0][2] - 1; k++) {
    for (j=1; j<pointNumberMeshU[0][1] - 1; j++) {
      u[0][0][j][k] = uAux[0][i][j][k] 
	- dt * (p[0][i][j-1][k-1] - p[0][pointNumberMeshP[0][0] - 1][j-1][k-1]) / h[0]; 
    }
  }
  
 
  //
  j = 0;
  for (k=1; k<pointNumberMeshV[0][2] - 1; k++) {
    for (i=1; i<pointNumberMeshV[0][0] - 1; i++) {
      v[0][i][j][k] = vAux[0][i][j][k] 
	- dt * (p[0][i-1][j][k-1] - p[0][i-1][pointNumberMeshP[0][1] - 1][k-1]) / h[0];  
    }
  }

  k = 0;
  for (j=1; j<pointNumberMeshV[0][1] - 1; j++) {
    for (i=1; i<pointNumberMeshV[0][0] - 1; i++) {
      w[0][i][j][k] = wAux[0][i][j][k] 
	- dt * (p[0][i-1][j-1][k] - p[0][i-1][j-1][pointNumberMeshP[0][2] - 1]) / h[0];  
    }
  }
  */


  ////////////////////////////////////
  //
  // outer boundary
  //
  ////////////////////////////////////

  boundaryCond b;
  b.initialize(2, // for uNew
	       t,
	       dt,
	       pointNumberMeshU, 
	       pointNumberMeshV, 
	       pointNumberMeshW, 
	       xU, 
	       xV, 
	       xW, 
	       u,
	       v,
	       w, 
	       uAux,
	       vAux,
	       wAux); 

}


uNew::~uNew(){
}
