#ifndef UNEW_H
#define UNEW_H

/** \file uNew.cpp
    \brief Calculate the velocity at the next step.

    This class is used to update the velocity at the next step.

*/

#include <iostream>
#include <string>
#include "boundaryCond.h"
#include "ghostPoint.h"
using namespace std;

#ifdef SINGLE
#define real float
#else
#define real double
#endif

/** 
    \brief Calculate the velocity at the next step.

    This class is used to update the velocity at the next step.

*/
class uNew {

 public:
  /**
     constructor
   */
  uNew();

/** 
    \brief Calculate the velocity at the next step.

    This class is used to update the velocity at the next step n+1. This is the last step in the projection method. 

*/
  void initialize(int numberOfPatches,/**< number of patches */
		  real t,/**<  time */
		  real dt,/**<  time increment */
		  int cellNumberPerPatch, /**<  cell numbers per patch surrounding the sphere*/
		  int ** StartIndex, /**< used to put the refined mesh */
		  int ** pointNumberMeshU, /**< mesh points for u */
		  int ** pointNumberMeshV, /**< mesh points for v */
		  int ** pointNumberMeshW, /**< mesh points for w */
		  int ** pointNumberMeshP, /**< mesh points for p */
		  int dimension,/**<  dimension */
		  real ***xU, /**< coordinates of u  */
		  real ***xV, /**< coordinates of v  */
		  real ***xW, /**< coordinates of w  */
		  real *h,/**< space step  */
		  real ****u,/**< velocity u */
		  int ****uCagePoint,/**< cage points for u */
		  real ****v,/**< velocity v */
		  int ****vCagePoint,/**< cage points for v */
		  real ****w,/**< velocity w */
		  int ****wCagePoint,/**< cage points for w */
		  real ****uAux,/**< predicted u */
		  real ****vAux,/**< predicted v */
		  real ****wAux,/**< predicted w */
		  real ****p/**<  pressure */
		  );
 
 /**
     destructor
   */
   ~uNew();
  
 private: 

  int nPatch; // each patch carries a square fine mesh
  int nMeshes;   // including the coarse mesh
  // the coarse mesh is the zeroth mesh and fine meshes are from 1st to the last
  int nCellPerPatch; // cell number in the coarse mesh that a patch uses

  int dim;


  // iRHS for index in matrix A = totalUnknowns x totalUnknowns
  // jRHS
  int i, j, k;


  // m for mesh iteration: meshes for patchs and coarse grid
  int m;



};

#endif
