#include "uStar.h"


uStar::uStar(){
}

void uStar::initialize(int numberOfPatches,
		       real t,
		       real dt,
		       real ReynoldsNumber,
		       int cellNumberPerPatch, 
		       int ** StartIndex, 
		       int ** pointNumberMeshU, 
		       int ** pointNumberMeshV, 
		       int ** pointNumberMeshW, 
		       int dimension,
		       real ***xU, 
		       real ***xV, 
		       real ***xW, 
		       real *h,
		       real ****u,
		       real ****uIter,
		       int ****uCagePoint,
		       real ****v,
		       real ****vIter,
		       int ****vCagePoint,
		       real ****w,
		       real ****wIter,
		       int ****wCagePoint,
		       real ****uAux,
		       real ****vAux,
		       real ****wAux)
{

  nPatch = numberOfPatches;
  nMeshes = numberOfPatches + 1;
  nCellPerPatch = cellNumberPerPatch;
  dim = dimension;
  Re = ReynoldsNumber;

  // enforce pressure griadient
  // BodyForce
  real fx = 0.0;
  real fy = 0.0;
  real fz = 0.0;
  
  ///////////////////////////////////////////
  //
  //interior points of u
  //
  //////////////////////////////////////////

  // from 2D
  // add [k] to u and v
  // copy those for uv
  // switch v and w
  // switch the formula of indices for j and k
  // for all variables of u, v, and w

  ///////////////////////////////////////////
  //
  //the coarse mesh
  //
  //////////////////////////////////////////

  for (k=1; k<pointNumberMeshU[0][2] - 1; k++) {
    for (j=1; j<pointNumberMeshU[0][1] - 1; j++) {
      for (i=1; i<pointNumberMeshU[0][0] - 1; i++) {
	uAux[0][i][j][k] = u[0][i][j][k] + dt
	  * ( ( pow(u[0][i-1][j][k]+u[0][i][j][k], 2.0) 
		- pow(u[0][i][j][k]+u[0][i+1][j][k], 2.0) ) / 4.0 / h[0]
	      + ( (u[0][i][j-1][k]+u[0][i][j][k]) * (v[0][i][j-1][k]+v[0][i+1][j-1][k]) 
		  -(u[0][i][j][k]+u[0][i][j+1][k]) * (v[0][i][j][k]+v[0][i+1][j][k]) ) 
	      / 4.0 / h[0]
	      + ( (u[0][i][j][k-1]+u[0][i][j][k]) * (w[0][i][j][k-1]+w[0][i+1][j][k-1]) 
		  -(u[0][i][j][k]+u[0][i][j][k+1]) * (w[0][i][j][k]+w[0][i+1][j][k]) ) 
	      / 4.0 / h[0]
	      +( u[0][i-1][j][k] - 2.0*u[0][i][j][k] + u[0][i+1][j][k]) / pow(h[0], 2.0) / Re
	      +( u[0][i][j-1][k] - 2.0*u[0][i][j][k] + u[0][i][j+1][k]) / pow(h[0], 2.0) / Re
	      +( u[0][i][j][k-1] - 2.0*u[0][i][j][k] + u[0][i][j][k+1]) / pow(h[0], 2.0) / Re 
	      + fx ); 
      }
    }
  }

  ///////////////////////////////////////////
  //
  //patchs
  //
  //////////////////////////////////////////

  for (m=1; m<=nPatch; m++) {
    for (k=1; k<pointNumberMeshU[m][2]-1; k++) {
      for (j=1; j<pointNumberMeshU[m][1]-1; j++) {
	for (i=1; i<pointNumberMeshU[m][0]-1; i++) {
	  uAux[m][i][j][k] = u[m][i][j][k] + dt
	    * ( ( pow(u[m][i-1][j][k]+u[m][i][j][k], 2.0) 
		  - pow(u[m][i][j][k]+u[m][i+1][j][k], 2.0) ) / 4.0 / h[m]
		+ ( (u[m][i][j-1][k]+u[m][i][j][k]) * (v[m][i-1][j][k]+v[m][i][j][k]) 
		    -(u[m][i][j][k]+u[m][i][j+1][k]) * (v[m][i-1][j+1][k]+v[m][i][j+1][k]) ) 
		/ 4.0 / h[m]
		+ ( (u[m][i][j][k-1]+u[m][i][j][k]) * (w[m][i-1][j][k]+w[m][i][j][k]) 
		    -(u[m][i][j][k]+u[m][i][j][k+1]) * (w[m][i-1][j][k+1]+w[m][i][j][k+1]) ) 
		/ 4.0 / h[m]
		+( u[m][i-1][j][k] - 2.0*u[m][i][j][k] + u[m][i+1][j][k]) / pow(h[m], 2.0) / Re
		+( u[m][i][j-1][k] - 2.0*u[m][i][j][k] + u[m][i][j+1][k]) / pow(h[m], 2.0) / Re
		+( u[m][i][j][k-1] - 2.0*u[m][i][j][k] + u[m][i][j][k+1]) / pow(h[m], 2.0) / Re
		+ fx );
	}
      }
    }
  }
  
  
  ///////////////////////////////////////////
  //
  //interior points of v
  //
  //////////////////////////////////////////

  // from 2D
  // add [k] to u and v
  // copy those for uv
  // switch w and u
  // switch the formula of indices for i and k
  // for all variables of u, v, and w

  // or,

  // from uAux
  // switch u and v
  // switch the formula of indices for i and j

  // or,

  // from wAux
  // switch w and v
  // switch the formula of indices for k and j

  ///////////////////////////////////////////
  //
  //the coarse mesh
  //
  //////////////////////////////////////////

  for (k=1; k<pointNumberMeshV[0][2] - 1; k++) {
    for (j=1; j<pointNumberMeshV[0][1] - 1; j++) {
      for (i=1; i<pointNumberMeshV[0][0] - 1; i++) {
	vAux[0][i][j][k] = v[0][i][j][k] + dt
	  * ( ( (u[0][i-1][j][k]+u[0][i-1][j+1][k]) * (v[0][i-1][j][k]+v[0][i][j][k]) 
		-(u[0][i][j][k]+u[0][i][j+1][k]) * (v[0][i][j][k]+v[0][i+1][j][k]) ) 
	      / 4.0 / h[0]
	      +( pow(v[0][i][j-1][k]+v[0][i][j][k], 2.0) 
		 - pow(v[0][i][j][k]+v[0][i][j+1][k], 2.0) ) / 4.0 / h[0]
	      +( (w[0][i][j][k-1]+w[0][i][j+1][k-1]) * (v[0][i][j][k-1]+v[0][i][j][k]) 
		 -(w[0][i][j][k]+w[0][i][j+1][k]) * (v[0][i][j][k]+v[0][i][j][k+1]) ) 
	      / 4.0 / h[0]
	      +( v[0][i-1][j][k] - 2.0*v[0][i][j][k] + v[0][i+1][j][k]) / pow(h[0], 2.0) / Re
	      +( v[0][i][j-1][k] - 2.0*v[0][i][j][k] + v[0][i][j+1][k]) / pow(h[0], 2.0) / Re
	      +( v[0][i][j][k-1] - 2.0*v[0][i][j][k] + v[0][i][j][k+1]) / pow(h[0], 2.0)  / Re
	      + fy ); 
      }
    }
  }
  
  ///////////////////////////////////////////
  //
  //patchs
  //
  //////////////////////////////////////////

  for (m=1; m<=nPatch; m++) {
    for (k=1; k<pointNumberMeshV[m][2]-1; k++) {
      for (j=1; j<pointNumberMeshV[m][1]-1; j++) {
	for (i=1; i<pointNumberMeshV[m][0]-1; i++) {
	  vAux[m][i][j][k] = v[m][i][j][k] + dt
	    * ( ( (u[m][i][j-1][k]+u[m][i][j][k]) * (v[m][i-1][j][k]+v[m][i][j][k]) 
		  -(u[m][i+1][j-1][k]+u[m][i+1][j][k]) * (v[m][i][j][k]+v[m][i+1][j][k]) ) 
		/ 4.0 / h[m]
		+( pow(v[m][i][j-1][k]+v[m][i][j][k], 2.0) 
		   - pow(v[m][i][j][k]+v[m][i][j+1][k], 2.0) ) / 4.0 / h[m]
		+( (w[m][i][j-1][k]+w[m][i][j][k]) * (v[m][i][j][k-1]+v[m][i][j][k]) 
		   -(w[m][i][j-1][k+1]+w[m][i][j][k+1]) * (v[m][i][j][k]+v[m][i][j][k+1]) ) 
		/ 4.0 / h[m]
		+( v[m][i-1][j][k] - 2.0*v[m][i][j][k] + v[m][i+1][j][k]) / pow(h[m], 2.0) / Re
		+( v[m][i][j-1][k] - 2.0*v[m][i][j][k] + v[m][i][j+1][k]) / pow(h[m], 2.0) / Re
		+( v[m][i][j][k-1] - 2.0*v[m][i][j][k] + v[m][i][j][k+1]) / pow(h[m], 2.0) / Re
		+ fy ); 
	}
      }
    }
  }
  
  ///////////////////////////////////////////
  //
  //interior points of w 
  //
  //////////////////////////////////////////

  // from uAux
  // switch u and w
  // switch the formula of indices for i and k

  // or,

  // from vAux
  // switch v and w
  // switch the formula of indices for j and k

  ///////////////////////////////////////////
  //
  //the coarse mesh
  //
  //////////////////////////////////////////

  for (k=1; k<pointNumberMeshW[0][2] - 1; k++) {
    for (j=1; j<pointNumberMeshW[0][1] - 1; j++) {
      for (i=1; i<pointNumberMeshW[0][0] - 1; i++) {
	wAux[0][i][j][k] = w[0][i][j][k] + dt
	  * ( ( (w[0][i-1][j][k]+w[0][i][j][k]) * (u[0][i-1][j][k]+u[0][i-1][j][k+1]) 
		-(w[0][i][j][k]+w[0][i+1][j][k]) * (u[0][i][j][k]+u[0][i][j][k+1]) ) 
	      / 4.0 / h[0]
	      + ( (w[0][i][j-1][k]+w[0][i][j][k]) * (v[0][i][j-1][k]+v[0][i][j-1][k+1]) 
		  -(w[0][i][j][k]+w[0][i][j+1][k]) * (v[0][i][j][k]+v[0][i][j][k+1]) ) 
	      / 4.0 / h[0]
	      + ( pow(w[0][i][j][k-1]+w[0][i][j][k], 2.0) 
		  - pow(w[0][i][j][k]+w[0][i][j][k+1], 2.0) ) / 4.0 / h[0]
	      + ( w[0][i-1][j][k] - 2.0*w[0][i][j][k] + w[0][i+1][j][k]) / pow(h[0], 2.0) / Re
	      + ( w[0][i][j-1][k] - 2.0*w[0][i][j][k] + w[0][i][j+1][k]) / pow(h[0], 2.0) / Re
	      + ( w[0][i][j][k-1] - 2.0*w[0][i][j][k] + w[0][i][j][k+1]) / pow(h[0], 2.0)  / Re
	      + fz ); 
      }
    }
  }
  
  ///////////////////////////////////////////
  //
  //patchs
  //
  //////////////////////////////////////////

  for (m=1; m<=nPatch; m++) {
    for (k=1; k<pointNumberMeshW[m][2]-1; k++) {
      for (j=1; j<pointNumberMeshW[m][1]-1; j++) {
	for (i=1; i<pointNumberMeshW[m][0]-1; i++) {
	  wAux[m][i][j][k] = w[m][i][j][k] + dt
	    * ( ( (u[m][i][j][k-1]+u[m][i][j][k]) * (w[m][i-1][j][k]+w[m][i][j][k]) 
		  -(u[m][i+1][j][k-1]+u[m][i+1][j][k]) * (w[m][i][j][k]+w[m][i+1][j][k]) ) 
		/ 4.0 / h[m]
		+( (v[m][i][j][k-1]+v[m][i][j][k]) * (w[m][i][j-1][k]+w[m][i][j][k]) 
		   -(v[m][i][j+1][k-1]+v[m][i][j+1][k]) * (w[m][i][j][k]+w[m][i][j+1][k]) ) 
		/ 4.0 / h[m]
		+( pow(w[m][i][j][k-1]+w[m][i][j][k], 2.0) 
		   - pow(w[m][i][j][k]+w[m][i][j][k+1], 2.0) ) / 4.0 / h[m]
		+( w[m][i-1][j][k] - 2.0*w[m][i][j][k] + w[m][i+1][j][k]) / pow(h[m], 2.0) / Re
		+( w[m][i][j][k-1] - 2.0*w[m][i][j][k] + w[m][i][j][k+1]) / pow(h[m], 2.0) / Re
		+( w[m][i][j-1][k] - 2.0*w[m][i][j][k] + w[m][i][j+1][k]) / pow(h[m], 2.0)  / Re
		+ fz ); 
	}
      }
    }
  }
  

  ///////////////////////////////////////////
  //
  // outer boundary for left u: periodic
  //
  //////////////////////////////////////////
  /*
  i = 0;
  for (k=1; k<pointNumberMeshU[0][2] - 1; k++) {
    for (j=1; j<pointNumberMeshU[0][1] - 1; j++) {
	uAux[0][i][j][k] = u[0][i][j][k] + dt
	  * ( ( pow(u[0][pointNumberMeshU[0][0] - 2][j][k]+u[0][i][j][k], 2.0) 
		- pow(u[0][i][j][k]+u[0][i+1][j][k], 2.0) ) / 4.0 / h[0]
	      + ( (u[0][i][j-1][k]+u[0][i][j][k]) * (v[0][i][j-1][k]+v[0][i+1][j-1][k]) 
		  -(u[0][i][j][k]+u[0][i][j+1][k]) * (v[0][i][j][k]+v[0][i+1][j][k]) ) 
	      / 4.0 / h[0]
	      + ( (u[0][i][j][k-1]+u[0][i][j][k]) * (w[0][i][j][k-1]+w[0][i+1][j][k-1]) 
		  -(u[0][i][j][k]+u[0][i][j][k+1]) * (w[0][i][j][k]+w[0][i+1][j][k]) ) 
	      / 4.0 / h[0]
	      +( u[0][pointNumberMeshU[0][0] - 2][j][k] - 2.0*u[0][i][j][k] + u[0][i+1][j][k]) / pow(h[0], 2.0) / Re
	      +( u[0][i][j-1][k] - 2.0*u[0][i][j][k] + u[0][i][j+1][k]) / pow(h[0], 2.0) / Re
	      +( u[0][i][j][k-1] - 2.0*u[0][i][j][k] + u[0][i][j][k+1]) / pow(h[0], 2.0) / Re 
	      + fx ); 
    }
  }
  

  ///////////////////////////////////////////
  //
  // outer boundary for south v: periodic
  //
  //////////////////////////////////////////
  j = 0;
  for (k=1; k<pointNumberMeshV[0][2] - 1; k++) {
    for (i=1; i<pointNumberMeshV[0][0] - 1; i++) {
      vAux[0][i][j][k] = v[0][i][j][k] + dt
	* ( ( (u[0][i-1][j][k]+u[0][i-1][j+1][k]) * (v[0][i-1][j][k]+v[0][i][j][k]) 
	      -(u[0][i][j][k]+u[0][i][j+1][k]) * (v[0][i][j][k]+v[0][i+1][j][k]) ) 
	    / 4.0 / h[0]
	    +( pow(v[0][i][pointNumberMeshV[0][1] - 2][k]+v[0][i][j][k], 2.0) 
	       - pow(v[0][i][j][k]+v[0][i][j+1][k], 2.0) ) / 4.0 / h[0]
	    +( (w[0][i][j][k-1]+w[0][i][j+1][k-1]) * (v[0][i][j][k-1]+v[0][i][j][k]) 
	       -(w[0][i][j][k]+w[0][i][j+1][k]) * (v[0][i][j][k]+v[0][i][j][k+1]) ) 
	    / 4.0 / h[0]
	    +( v[0][i-1][j][k] - 2.0*v[0][i][j][k] + v[0][i+1][j][k]) / pow(h[0], 2.0) / Re
	    +( v[0][i][pointNumberMeshV[0][1] - 2][k] - 2.0*v[0][i][j][k] + v[0][i][j+1][k]) / pow(h[0], 2.0) / Re
	    +( v[0][i][j][k-1] - 2.0*v[0][i][j][k] + v[0][i][j][k+1]) / pow(h[0], 2.0)  / Re
	    + fy ); 
    }
  }
  


  ///////////////////////////////////////////
  //
  // outer boundary for bottom w: periodic
  //
  //////////////////////////////////////////
  k = 0;
  for (j=1; j<pointNumberMeshW[0][1] - 1; j++) {
    for (i=1; i<pointNumberMeshW[0][0] - 1; i++) {
      wAux[0][i][j][k] = w[0][i][j][k] + dt
	* ( ( (w[0][i-1][j][k]+w[0][i][j][k]) * (u[0][i-1][j][k]+u[0][i-1][j][k+1]) 
	      -(w[0][i][j][k]+w[0][i+1][j][k]) * (u[0][i][j][k]+u[0][i][j][k+1]) ) 
	    / 4.0 / h[0]
	    + ( (w[0][i][j-1][k]+w[0][i][j][k]) * (v[0][i][j-1][k]+v[0][i][j-1][k+1]) 
		-(w[0][i][j][k]+w[0][i][j+1][k]) * (v[0][i][j][k]+v[0][i][j][k+1]) ) 
	    / 4.0 / h[0]
	    + ( pow(w[0][i][j][pointNumberMeshW[0][2] - 2]+w[0][i][j][k], 2.0) 
		- pow(w[0][i][j][k]+w[0][i][j][k+1], 2.0) ) / 4.0 / h[0]
	    + ( w[0][i-1][j][k] - 2.0*w[0][i][j][k] + w[0][i+1][j][k]) / pow(h[0], 2.0) / Re
	    + ( w[0][i][j-1][k] - 2.0*w[0][i][j][k] + w[0][i][j+1][k]) / pow(h[0], 2.0) / Re
	    + ( w[0][i][j][pointNumberMeshW[0][2] - 2] - 2.0*w[0][i][j][k] + w[0][i][j][k+1]) / pow(h[0], 2.0)  / Re
	    + fz ); 
    }
  }
  */


  ///////////////////////////////////////////
  //
  //outer boundary of other sides
  //
  //////////////////////////////////////////

  boundaryCond b;
  b.initialize(1, // for uStar
	       t,
	       dt,
	       pointNumberMeshU, 
	       pointNumberMeshV, 
	       pointNumberMeshW, 
	       xU, 
	       xV, 
	       xW, 
	       u,
	       v,
	       w, 
	       uAux,
	       vAux,
	       wAux);


  ///////////////////////////////////////////
  //
  // inner boundary of particles 
  //
  //////////////////////////////////////////

  for (m=1; m<=nPatch; m++) {
    for (k=0; k<pointNumberMeshU[m][2]; k++) {
      for (j=0; j<pointNumberMeshU[m][1]; j++) {
	for (i=0; i<pointNumberMeshU[m][0]; i++) {
	  
	  if (uCagePoint[m][i][j][k] == 1){
	    uAux[m][i][j][k] = uIter[m][i][j][k];
	  }
	  
	}   
      }
    }
  }

  for (m=1; m<=nPatch; m++) {
    for (k=0; k<pointNumberMeshV[m][2]; k++) {
      for (j=0; j<pointNumberMeshV[m][1]; j++) {
	for (i=0; i<pointNumberMeshV[m][0]; i++) {
	  
	  if (vCagePoint[m][i][j][k] == 1){
	    vAux[m][i][j][k] = vIter[m][i][j][k];
	  }
	  
	}
      }
    }
  }

  for (m=1; m<=nPatch; m++) {
    for (k=0; k<pointNumberMeshW[m][2]; k++) {
      for (j=0; j<pointNumberMeshW[m][1]; j++) {
	for (i=0; i<pointNumberMeshW[m][0]; i++) {
	  
	  if (wCagePoint[m][i][j][k] == 1){
	    wAux[m][i][j][k] = wIter[m][i][j][k];
	  }
	  
	}
      }
    }
  }


}



uStar::~uStar(){
}
