#ifndef USTAR_H
#define USTAR_H

/** \file uStar.cpp
    \brief Calculate the predicted velocity u*.

    This class is used to calculate the predicted velocity u*.
*/
#include <iostream>
#include <string>
#include <cmath>
#include "boundaryCond.h"
#include "ghostPoint.h"
using namespace std;

#ifdef SINGLE
#define real float
#else
#define real double
#endif

/** 
    \brief Calculate the predicted velocity u*.

    This class is used to calculate the predicted velocity u*.
*/
class uStar {

 public:
  /**
     constructor
   */
  uStar();

/** 
    \brief Calculate the predicted velocity u*.

    Based on the given velocity field in the time step n, this class is used to calculate the predicted velocity u*, which is the first step in the projection meth.
*/  
  void initialize(int numberOfPatches,/**< number of patches */
		  real t, /**<  time */
		  real dt, /**<  time increment */
		  real ReynoldsNumber,/**<  Reynolds number */
		  int cellNumberPerPatch, /**<  cell numbers per patch surrounding the sphere*/
		  int ** StartIndex, /**< used to put the refined mesh */
		  int ** pointNumberMeshU, /**< mesh points for u */
		  int ** pointNumberMeshV, /**< mesh points for v */
		  int ** pointNumberMeshW, /**< mesh points for w */
		  int dimension, /**<  dimension */
		  real ***xU, /**< coordinates of u  */
		  real ***xV, /**< coordinates of v  */
		  real ***xW, /**< coordinates of w  */
		  real *h,/**< space step  */
		  real ****u,/**< velocity u */
		  real ****uIter,/**< velocity u in the iteration*/
		  int ****uCagePoint,/**< cage points for u */
		  real ****v,/**< velocity v */
		  real ****vIter,/**< velocity v in the iteration*/
		  int ****vCagePoint,/**< cage points for v */
		  real ****w,/**< velocity w */
		  real ****wIter,/**< velocity w in the iteration*/
		  int ****wCagePoint,/**< cage points for w */
		  real ****uAux,/**< predicted u */
		  real ****vAux,/**< predicted v */
		  real ****wAux /**< predicted w */
		  );

  /**
     destructor
   */
   ~uStar();
  
 private: 


  int nPatch; // each patch carries a square fine mesh
  int nMeshes;   // including the coarse mesh
  // the coarse mesh is the zeroth mesh and fine meshes are from 1st to the last
  int nCellPerPatch; // cell number in the coarse mesh that a patch uses

  int dim;

  real Re; // Reynolds number based on the radius of a particle

  // iRHS for index in matrix A = totalUnknowns x totalUnknowns
  // jRHS
  int i, j, k;


  // m for mesh iteration: meshes for patchs and coarse grid
  int m;



};

#endif
