#ifndef VORTICITY_H
#define VORTICITY_H

/** \file vorticity.cpp
    \brief Calculate vorticity

    This class is used to calculate vorticity.
*/


#include <iostream>
#include <string>
using namespace std;

#ifdef SINGLE
#define real float
#else
#define real double
#endif


/** 
    \brief Calculate vorticity

    This class is used to calculate vorticity from the known velocity field.
*/


class vorticity {

 public:
  /**
     constructor
   */
  vorticity();

/** 
    \brief Calculate vorticity

    This class is used to calculate vorticity. The vorticity information is needed by the Physalis method to impose the no-slip boundary conditions on the sphere.
*/
  void calOmegaFromUVW(int numberOfPatches,/**< number of patches */
		       int ** pointNumberMeshOmegaX, /**< mesh points for vorticity omegaX */
		       int ** pointNumberMeshOmegaY, /**< mesh points for vorticity omegaY */
		       int ** pointNumberMeshOmegaZ, /**< mesh points for vorticity omegaZ */
		       real *h,/**< space step  */
		       real ****u,/**< velocity u */
		       real ****v,/**< velocity v */
		       real ****w,/**< velocity w */
		       real ****omegaX, /**< vorticity omegaX */
		       real ****omegaY,/**< vorticity omegaY */
		       real ****omegaZ/**< vorticity omegaZ */
		       );

  /**
     destructor
   */
  ~vorticity();
  
 private: 

  int nPatch; // each patch carries a square fine mesh
  // the coarse mesh is the zeroth mesh and fine meshes are from 1st to the last

  // iRHS for index in matrix A = totalUnknowns x totalUnknowns
  // jRHS
  int i, j, k;


  // m for mesh iteration: meshes for patchs and coarse grid
  int m;



};

#endif
